import torch
import torch.nn as nn
import torch.nn.functional as F
import geffnet
import numpy as np


INPUT_CHANNELS_DICT = {
    0: [1280, 112, 40, 24, 16],
    1: [1280, 112, 40, 24, 16],
    2: [1408, 120, 48, 24, 16],
    3: [1536, 136, 48, 32, 24],
    4: [1792, 160, 56, 32, 24],
    5: [2048, 176, 64, 40, 24],
    6: [2304, 200, 72, 40, 32],
    7: [2560, 224, 80, 48, 32],
}


class Encoder(nn.Module):
    def __init__(self, B=5, pretrained=True):
        """e.g. B=5 will return EfficientNet-B5"""
        super(Encoder, self).__init__()
        basemodel = geffnet.create_model(
            "tf_efficientnet_b%s_ap" % B, pretrained=pretrained
        )
        # Remove last layer
        basemodel.global_pool = nn.Identity()
        basemodel.classifier = nn.Identity()
        self.original_model = basemodel

    def forward(self, x):
        features = [x]
        for k, v in self.original_model._modules.items():
            if k == "blocks":
                for ki, vi in v._modules.items():
                    features.append(vi(features[-1]))
            else:
                features.append(v(features[-1]))
        return features


class ConvGRU(nn.Module):
    def __init__(self, hidden_dim, input_dim, ks=3):
        super(ConvGRU, self).__init__()
        p = (ks - 1) // 2
        self.convz = nn.Conv2d(hidden_dim + input_dim, hidden_dim, ks, padding=p)
        self.convr = nn.Conv2d(hidden_dim + input_dim, hidden_dim, ks, padding=p)
        self.convq = nn.Conv2d(hidden_dim + input_dim, hidden_dim, ks, padding=p)

    def forward(self, h, x):
        hx = torch.cat([h, x], dim=1)
        z = torch.sigmoid(self.convz(hx))
        r = torch.sigmoid(self.convr(hx))
        q = torch.tanh(self.convq(torch.cat([r * h, x], dim=1)))
        h = (1 - z) * h + z * q
        return h


class RayReLU(nn.Module):
    def __init__(self, eps=1e-2):
        super(RayReLU, self).__init__()
        self.eps = eps

    def forward(self, pred_norm, ray):
        # angle between the predicted normal and ray direction
        cos = torch.cosine_similarity(pred_norm, ray, dim=1).unsqueeze(
            1
        )  # (B, 1, H, W)

        # component of pred_norm along view
        norm_along_view = ray * cos

        # cos should be bigger than eps
        norm_along_view_relu = ray * (torch.relu(cos - self.eps) + self.eps)

        # difference
        diff = norm_along_view_relu - norm_along_view

        # updated pred_norm
        new_pred_norm = pred_norm + diff
        new_pred_norm = F.normalize(new_pred_norm, dim=1)

        return new_pred_norm


class UpSampleBN(nn.Module):
    def __init__(self, skip_input, output_features, align_corners=True):
        super(UpSampleBN, self).__init__()
        self._net = nn.Sequential(
            nn.Conv2d(skip_input, output_features, kernel_size=3, stride=1, padding=1),
            nn.BatchNorm2d(output_features),
            nn.LeakyReLU(),
            nn.Conv2d(
                output_features, output_features, kernel_size=3, stride=1, padding=1
            ),
            nn.BatchNorm2d(output_features),
            nn.LeakyReLU(),
        )
        self.align_corners = align_corners

    def forward(self, x, concat_with):
        up_x = F.interpolate(
            x,
            size=[concat_with.size(2), concat_with.size(3)],
            mode="bilinear",
            align_corners=self.align_corners,
        )
        f = torch.cat([up_x, concat_with], dim=1)
        return self._net(f)


class Conv2d_WS(nn.Conv2d):
    """weight standardization"""

    def __init__(
        self,
        in_channels,
        out_channels,
        kernel_size,
        stride=1,
        padding=0,
        dilation=1,
        groups=1,
        bias=True,
    ):
        super(Conv2d_WS, self).__init__(
            in_channels,
            out_channels,
            kernel_size,
            stride,
            padding,
            dilation,
            groups,
            bias,
        )

    def forward(self, x):
        weight = self.weight
        weight_mean = (
            weight.mean(dim=1, keepdim=True)
            .mean(dim=2, keepdim=True)
            .mean(dim=3, keepdim=True)
        )
        weight = weight - weight_mean
        std = weight.view(weight.size(0), -1).std(dim=1).view(-1, 1, 1, 1) + 1e-5
        weight = weight / std.expand_as(weight)
        return F.conv2d(
            x, weight, self.bias, self.stride, self.padding, self.dilation, self.groups
        )


class UpSampleGN(nn.Module):
    """UpSample with GroupNorm"""

    def __init__(self, skip_input, output_features, align_corners=True):
        super(UpSampleGN, self).__init__()
        self._net = nn.Sequential(
            Conv2d_WS(skip_input, output_features, kernel_size=3, stride=1, padding=1),
            nn.GroupNorm(8, output_features),
            nn.LeakyReLU(),
            Conv2d_WS(
                output_features, output_features, kernel_size=3, stride=1, padding=1
            ),
            nn.GroupNorm(8, output_features),
            nn.LeakyReLU(),
        )
        self.align_corners = align_corners

    def forward(self, x, concat_with):
        up_x = F.interpolate(
            x,
            size=[concat_with.size(2), concat_with.size(3)],
            mode="bilinear",
            align_corners=self.align_corners,
        )
        f = torch.cat([up_x, concat_with], dim=1)
        return self._net(f)


def upsample_via_bilinear(out, up_mask, downsample_ratio):
    """bilinear upsampling (up_mask is a dummy variable)"""
    return F.interpolate(
        out, scale_factor=downsample_ratio, mode="bilinear", align_corners=True
    )


def upsample_via_mask(out, up_mask, downsample_ratio, padding="zero"):
    """convex upsampling"""
    # out: low-resolution output (B, o_dim, H, W)
    # up_mask: (B, 9*k*k, H, W)
    k = downsample_ratio

    B, C, H, W = out.shape
    up_mask = up_mask.view(B, 1, 9, k, k, H, W)
    up_mask = torch.softmax(up_mask, dim=2)  # (B, 1, 9, k, k, H, W)

    if padding == "zero":
        # with zero padding
        up_out = F.unfold(out, [3, 3], padding=1)  # (B, 2, H, W) -> (B, 2 X 3*3, H*W)
    elif padding == "replicate":
        # with replicate padding
        out = F.pad(out, pad=(1, 1, 1, 1), mode="replicate")
        up_out = F.unfold(out, [3, 3], padding=0)  # (B, C, H, W) -> (B, C X 3*3, H*W)
    else:
        raise Exception("invalid padding for convex upsampling")

    up_out = up_out.view(B, C, 9, 1, 1, H, W)  # (B, C, 9, 1, 1, H, W)

    up_out = torch.sum(up_mask * up_out, dim=2)  # (B, C, k, k, H, W)
    up_out = up_out.permute(0, 1, 4, 2, 5, 3)  # (B, C, H, k, W, k)
    return up_out.reshape(B, C, k * H, k * W)  # (B, C, kH, kW)


def convex_upsampling(out, up_mask, k):
    # out: low-resolution output    (B, C, H, W)
    # up_mask:                      (B, 9*k*k, H, W)
    B, C, H, W = out.shape
    up_mask = up_mask.view(B, 1, 9, k, k, H, W)
    up_mask = torch.softmax(up_mask, dim=2)  # (B, 1, 9, k, k, H, W)

    out = F.pad(out, pad=(1, 1, 1, 1), mode="replicate")
    up_out = F.unfold(out, [3, 3], padding=0)  # (B, C, H, W) -> (B, C X 3*3, H*W)
    up_out = up_out.view(B, C, 9, 1, 1, H, W)  # (B, C, 9, 1, 1, H, W)

    up_out = torch.sum(up_mask * up_out, dim=2)  # (B, C, k, k, H, W)
    up_out = up_out.permute(0, 1, 4, 2, 5, 3)  # (B, C, H, k, W, k)
    return up_out.reshape(B, C, k * H, k * W)  # (B, C, kH, kW)


def get_unfold(pred_norm, ps, pad):
    B, C, H, W = pred_norm.shape
    pred_norm = F.pad(
        pred_norm, pad=(pad, pad, pad, pad), mode="replicate"
    )  # (B, C, h, w)
    pred_norm_unfold = F.unfold(pred_norm, [ps, ps], padding=0)  # (B, C X ps*ps, h*w)
    pred_norm_unfold = pred_norm_unfold.view(B, C, ps * ps, H, W)  # (B, C, ps*ps, h, w)
    return pred_norm_unfold


def get_prediction_head(input_dim, hidden_dim, output_dim):
    return nn.Sequential(
        nn.Conv2d(input_dim, hidden_dim, 3, padding=1),
        nn.ReLU(inplace=True),
        nn.Conv2d(hidden_dim, hidden_dim, 1),
        nn.ReLU(inplace=True),
        nn.Conv2d(hidden_dim, output_dim, 1),
    )


def normal_activation(out, elu_kappa=True):
    normal, kappa = out[:, :3, :, :], out[:, 3:, :, :]
    normal = F.normalize(normal, p=2, dim=1)
    if elu_kappa:
        kappa = F.elu(kappa) + 1.0
    return torch.cat([normal, kappa], dim=1)


def get_pixel_coords(h, w):
    # pixel array (1, 2, H, W)
    pixel_coords = np.ones((3, h, w)).astype(np.float32)
    x_range = np.concatenate([np.arange(w).reshape(1, w)] * h, axis=0)
    y_range = np.concatenate([np.arange(h).reshape(h, 1)] * w, axis=1)
    pixel_coords[0, :, :] = x_range + 0.5
    pixel_coords[1, :, :] = y_range + 0.5
    return torch.from_numpy(pixel_coords).unsqueeze(0)



from typing import Literal
import torch
import numpy as np
from jaxtyping import Float, UInt8
from timeit import default_timer as timer
from huggingface_hub import hf_hub_download
from monopriors.depth_utils import estimate_intrinsics, disparity_to_depth
from monopriors.third_party.depth_anything_v2.dpt import DepthAnythingV2
import cv2
from jaxtyping import Float32
from .base_relative_depth import RelativeDepthPrediction, BaseRelativePredictor

model_configs = {
    "vits": {
        "encoder": "vits",
        "features": 64,
        "out_channels": [48, 96, 192, 384],
    },
    "vitb": {
        "encoder": "vitb",
        "features": 128,
        "out_channels": [96, 192, 384, 768],
    },
    "vitl": {
        "encoder": "vitl",
        "features": 256,
        "out_channels": [256, 512, 1024, 1024],
    },
    "vitg": {
        "encoder": "vitg",
        "features": 384,
        "out_channels": [1536, 1536, 1536, 1536],
    },
}
encoder2name: dict[str, str] = {
    "vits": "Small",
    "vitb": "Base",
    "vitl": "Large",
    "vitg": "Giant",
}


class DepthAnythingV2Predictor(BaseRelativePredictor):
    def __init__(
        self,
        device: Literal["cpu", "cuda"],
        encoder: Literal["vits", "vitb", "vitl"] = "vits",
    ) -> None:
        super().__init__()
        print("Loading DepthAnythingV2 model...")
        start = timer()
        model_name: str = encoder2name[encoder]
        self.model = DepthAnythingV2(**model_configs[encoder])
        filepath: str = hf_hub_download(
            repo_id=f"depth-anything/Depth-Anything-V2-{model_name}",
            filename=f"depth_anything_v2_{encoder}.pth",
            repo_type="model",
        )
        state_dict = torch.load(filepath, map_location="cpu")
        self.model.load_state_dict(state_dict)
        self.model = self.model.to(device).eval()
        print(f"DepthAnythingV2 model loaded. Time: {timer() - start:.2f}s")

    def __call__(
        self, rgb: UInt8[np.ndarray, "h w 3"], K_33: Float[np.ndarray, "3 3"] | None
    ) -> RelativeDepthPrediction:
        # requires bgr not rgb
        bgr: UInt8[np.ndarray, "h w 3"] = cv2.cvtColor(rgb, cv2.COLOR_RGB2BGR)
        disparity: Float32[np.ndarray, "h w"] = self.model.infer_image(bgr)

        if K_33 is None:
            K_33 = estimate_intrinsics(rgb.shape[0], rgb.shape[1])

        relative_prediction = RelativeDepthPrediction(
            disparity=disparity,
            depth=disparity_to_depth(disparity, focal_length=int(K_33[0, 0])),
            confidence=np.ones_like(disparity),
            K_33=K_33,
        )

        return relative_prediction



from .base_relative_depth import RelativeDepthPrediction, BaseRelativePredictor
from .depth_anything_v2 import DepthAnythingV2Predictor
from .depth_anything_v1 import DepthAnythingV1Predictor
from .unidepth import UniDepthRelativePredictor
from .metric3d_relative import Metric3DRelativePredictor
from typing import Literal, get_args, Callable

# Define predictor names as a list of strings
RELATIVE_PREDICTORS = Literal[
    "DepthAnythingV1Predictor",
    "DepthAnythingV2Predictor",
    "UniDepthRelativePredictor",
    "Metric3DRelativePredictor",
]

# Use the list to generate the __all__ list
__all__: list[str] = list(get_args(RELATIVE_PREDICTORS)) + [
    "RelativeDepthPrediction",
]


def get_relative_predictor(
    predictor_type: RELATIVE_PREDICTORS,
) -> Callable[..., BaseRelativePredictor]:
    match predictor_type:
        case "UniDepthRelativePredictor":
            return UniDepthRelativePredictor
        case "DepthAnythingV2Predictor":
            return DepthAnythingV2Predictor
        case "DepthAnythingV1Predictor":
            return DepthAnythingV1Predictor
        case "Metric3DRelativePredictor":
            return Metric3DRelativePredictor
        case _:
            raise ValueError(f"Unknown predictor type: {predictor_type}")



import argparse
import json
import os
from pathlib import Path

import cv2
import matplotlib.pyplot as plt
import numpy as np
from PIL import Image
from tqdm import tqdm

from monopriors.data.nerfstudio_data import load_nerfstudio_from_json
from monopriors.data.sdfstudio_data import SDFStudioFrame, SceneBox, SDFStudioData
from monopriors.monoprior_models import DsineAndUnidepth, OldMonoPriorPrediction
import rerun as rr
from icecream import ic
from jaxtyping import Bool, Float64
from typing import Literal

from dataclasses import asdict

import calibur


def main(
    input_dir: Path,
    output_dir: Path,
    scene_type: Literal["indoor", "object"],
    log=False,
):
    """
    given data that follows the nerfstduio format such as the output from colmap or polycam, convert to a format
    that sdfstudio will ingest
    """

    output_dir.mkdir(parents=True, exist_ok=True)

    # load transformation json with images/intrinsics/extrinsics
    camera_parameters_path = input_dir / "transforms.json"
    ns_data = load_nerfstudio_from_json(camera_parameters_path)

    ns_log_path = Path("ns")

    world_T_cam_cv_44_list = []
    image_paths = []
    K_33_list = []
    for idx, frame in enumerate(ns_data.frames):
        image_path = input_dir / frame.file_path
        bgr = cv2.imread(str(image_path))
        rgb = cv2.cvtColor(bgr, cv2.COLOR_BGR2RGB)

        world_T_cam_gl_44 = np.array(frame.transform_matrix).reshape(4, 4)
        K_33 = np.array(
            [[frame.fl_x, 0, frame.cx], [0, frame.fl_y, frame.cy], [0, 0, 1]]
        )
        K_33_list.append(K_33)

        # if log and idx % 10 == 0:
        #     cam_log_path = ns_log_path / f"cam_{idx:03d}"
        #     pinhole_log_path = cam_log_path / "pinhole"
        #     rr.log(
        #         str(cam_log_path),
        #         rr.Transform3D(
        #             translation=world_T_cam_gl_44[:3, 3],
        #             mat3x3=world_T_cam_gl_44[:3, :3],
        #             from_parent=False,
        #         ),
        #     )
        #     rr.log(
        #         str(pinhole_log_path),
        #         rr.Pinhole(
        #             image_from_camera=K_33,
        #             width=frame.w,
        #             height=frame.h,
        #             camera_xyz=rr.ViewCoordinates.RUB,
        #         ),
        #     )
        #     rr.log(
        #         f"{pinhole_log_path}/image",
        #         rr.Image(rgb),
        #     )

        height, width, _ = rgb.shape

        world_T_cam_cv_44 = calibur.convert_pose(
            world_T_cam_gl_44,
            src_convention=calibur.CC.GL,
            dst_convention=calibur.CC.CV,
        )
        world_T_cam_cv_44_list.append(world_T_cam_cv_44)
        image_paths.append(image_path)
        assert image_path.exists()

    world_T_cam_cv_b44 = np.array(world_T_cam_cv_44_list)
    K_b33 = np.array(K_33_list)

    valid_poses: Bool[np.ndarray, ""] = (
        np.isfinite(world_T_cam_cv_b44).all(axis=2).all(axis=1)
    )
    min_vertices: Float64[np.ndarray, "3"] = world_T_cam_cv_b44[:, :3, 3][
        valid_poses
    ].min(axis=0)
    max_vertices: Float64[np.ndarray, "3"] = world_T_cam_cv_b44[:, :3, 3][
        valid_poses
    ].max(axis=0)

    # Perform Camera normalization
    if scene_type == "indoor":
        scene_center = (min_vertices + max_vertices) / 2.0
        scene_scale = 2.0 / (np.max(max_vertices - min_vertices) + 3.0)

        # normalize pose to unit cube
        world_T_cam_cv_b44[:, :3, 3] -= scene_center
        world_T_cam_cv_b44[:, :3, 3] *= scene_scale

        # inverse normalization to recover original poses
        world_T_gt = np.eye(4).astype(np.float32)
        world_T_gt[:3, 3] -= scene_center
        world_T_gt[:3] *= scene_scale
        gt_T_world = np.linalg.inv(world_T_gt)
    else:
        gt_T_world = np.eye(4).astype(np.float32)

    # Generate SDFStudio data json and save related images/monopriors
    model = DsineAndUnidepth()

    frames = []
    sdf_log_path = Path("sdf")
    for idx, (valid, world_T_cam_cv_44, image_path) in enumerate(
        tqdm(zip(valid_poses, world_T_cam_cv_b44, image_paths))
    ):
        if not valid:
            continue

        # save rgb image
        target_image = output_dir / f"{idx:06d}_rgb.png"
        bgr = cv2.imread(str(image_path))
        rgb = cv2.cvtColor(bgr, cv2.COLOR_BGR2RGB)
        cv2.imwrite(str(target_image), rgb)
        # img = Image.open(image_path)
        # img.save(target_image)

        K_33 = K_b33[idx]
        pred: OldMonoPriorPrediction = model(rgb, K_33.astype(np.float32))
        depth_np_bhw1, normal_np_bhw3, _ = pred.to_numpy()

        # convert to normalized like omnidata (this is what nerfstudio accepts)
        normal_np_bhw3[..., 2] *= -1
        normal_np_bhw3 = (normal_np_bhw3 + 1.0) / 2.0

        rgb_path = str(target_image.relative_to(output_dir))

        # save depth and normal using numpy
        mono_depth_path = rgb_path.replace("_rgb.png", "_depth.npy")
        mono_normal_path = rgb_path.replace("_rgb.png", "_normal.npy")

        # convert hw1 depth to hw3 using grayscale to rgb
        depth_np_hw1 = depth_np_bhw1[0]
        # normalize depth to 0-1
        depth_np_hw1 = (depth_np_hw1 - depth_np_hw1.min()) / (
            depth_np_hw1.max() - depth_np_hw1.min()
        )
        depth_np_hw = depth_np_hw1.squeeze()
        np.save(output_dir / mono_depth_path, depth_np_hw)
        np.save(output_dir / mono_normal_path, normal_np_bhw3[0])
        # save as images
        plt.imsave(
            output_dir / mono_depth_path.replace(".npy", ".png"),
            depth_np_hw,
            cmap="viridis",
        )
        plt.imsave(
            output_dir / mono_normal_path.replace(".npy", ".png"), normal_np_bhw3[0]
        )

        frame = SDFStudioFrame(
            rgb_path=rgb_path,
            camtoworld=world_T_cam_cv_44.tolist(),
            intrinsics=K_33.tolist(),
            mono_depth_path=mono_depth_path,
            mono_normal_path=mono_normal_path,
        )

        frames.append(frame)
        rr.set_time_sequence("timestep", idx)
        if log:
            cam_log_path = sdf_log_path / "cam"
            pinhole_log_path = cam_log_path / "pinhole"
            rr.log(
                str(cam_log_path),
                rr.Transform3D(
                    translation=world_T_cam_cv_44[:3, 3],
                    mat3x3=world_T_cam_cv_44[:3, :3],
                    from_parent=False,
                ),
            )
            rr.log(
                str(pinhole_log_path),
                rr.Pinhole(
                    image_from_camera=K_b33[idx],
                    width=width,
                    height=height,
                    camera_xyz=rr.ViewCoordinates.RDF,
                ),
            )
            rr.log(
                f"{pinhole_log_path}/image",
                rr.Image(rgb),
            )
            rr.log(
                f"{pinhole_log_path}/depth",
                rr.DepthImage(depth_np_bhw1[0]),
            )
            rr.log(
                f"{pinhole_log_path}/normal",
                rr.Image(normal_np_bhw3[0]),
            )

    # scene bbox for the scannet scene
    if scene_type == "indoor":
        scene_box = SceneBox(
            aabb=[[-1, -1, -1], [1, 1, 1]],
            near=0.05,
            far=2.5,
            radius=1.0,
            collider_type="box",
        )
    else:
        raise NotImplementedError

    # meta data
    sdf_data = SDFStudioData(
        camera_model="OPENCV",
        height=height,
        width=width,
        has_mono_prior=True,
        worldtogt=gt_T_world.tolist(),
        scene_box=scene_box,
        frames=frames,
        pairs=None,
    )

    sdf_dict = asdict(sdf_data)

    # save as json
    with open(output_dir / "meta_data.json", "w", encoding="utf-8") as f:
        json.dump(sdf_dict, f, indent=4)


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="preprocess scannet dataset to sdfstudio dataset"
    )

    parser.add_argument(
        "--input-dir", help="path to polycam data directory", required=True, type=Path
    )
    parser.add_argument(
        "--output-dir", help="path to output directory", required=True, type=Path
    )
    parser.add_argument(
        "--scene-type",
        dest="scene_type",
        required=True,
        choices=["indoor", "object"],
    )
    parser.add_argument(
        "--log", help="log to rerun", action="store_true", default=False
    )
    rr.script_add_args(parser)
    args = parser.parse_args()
    rr.script_setup(args, "polycam to sdfstudio")
    main(args.input_dir, args.output_dir, args.scene_type, log=args.log)
    rr.script_teardown(args)



import subprocess
from pathlib import Path

PIXI_PATH = Path("/home/user/.pixi/bin/pixi")
BREW_PATH = Path("/home/linuxbrew/.linuxbrew/bin/brew")
LSOF_PATH = Path("/home/linuxbrew/.linuxbrew/Cellar/lsof/4.99.3/bin/lsof")


def check_and_install_pixi() -> None:
    try:
        subprocess.check_call(f"{PIXI_PATH} --version", shell=True)
    except subprocess.CalledProcessError:
        print("pixi not found. Installing pixi...")
        # Install pixi using the provided installation script
        subprocess.check_call(
            "curl -fsSL https://pixi.sh/install.sh | bash", shell=True
        )


def check_and_install_homebrew() -> None:
    try:
        # Check if Homebrew is installed
        subprocess.check_call(f"{BREW_PATH} --version", shell=True)
    except subprocess.CalledProcessError:
        # If Homebrew is not found, install it
        print("Homebrew not found. Installing Homebrew...")
        subprocess.check_call(
            '/bin/bash -c "$(curl -fsSL https://raw.githubusercontent.com/Homebrew/install/HEAD/install.sh)"',
            shell=True,
        )


def install_package(package_name) -> None:
    try:
        # Install the specified package using Homebrew
        subprocess.check_call(f"{BREW_PATH} install {package_name}", shell=True)
        print(f"{package_name} installed successfully.")
    except subprocess.CalledProcessError as e:
        print(f"Failed to install {package_name}. Error: {e}")


def run_command(command: str) -> None:
    try:
        subprocess.check_call(command, shell=True)
    except subprocess.CalledProcessError as e:
        print(f"run command {command}. Error: {e}")


if __name__ == "__main__":
    check_and_install_homebrew()
    install_package(package_name="lsof")
    check_and_install_pixi()
    run_command(command=f"{LSOF_PATH} -t -i:7860 | xargs -r kill")
    run_command(command=f"{PIXI_PATH} run -e spaces app")



from argparse import ArgumentParser
import rerun as rr
from pathlib import Path
from PIL import Image
from monopriors.normal_models import DSineNormalPredictor, OmniNormalPredictor
import numpy as np
import zipfile


def log_compare_normals(
    parent_path,
    rgb,
    normal_np_1,
    normal_np_2,
):
    cam_path = parent_path / "cam"
    pinhole_path = cam_path / "pinhole"

    h, w, _ = rgb.shape
    # move from 0 -1 to -1 1
    normal_np_1 = (normal_np_1 - 0.5) * 2
    # convert from h,w,c to hw, c
    normal_np_1 = normal_np_1.reshape(-1, 3)
    # convert from OpenCV coordinate system (RDF) to OpenGL coordinate system (RUB)
    # convert normal map from opengl to opencv
    rotation = np.diag([1, -1, -1])
    normal_np_1 = normal_np_1 @ rotation

    normal_np_1 = normal_np_1.reshape(h, w, 3)

    # convert normal map 2 from LUB to RUB
    normal_np_2 = normal_np_2.reshape(-1, 3)
    rotation = np.diag([-1, 1, 1])
    normal_np_2 = normal_np_2 @ rotation
    normal_np_2 = normal_np_2.reshape(h, w, 3)

    rr.log(f"{pinhole_path}/image", rr.Image(rgb))
    rr.log(f"{pinhole_path}/normal_1", rr.Image(normal_np_1))
    rr.log(f"{pinhole_path}/normal_2", rr.Image(normal_np_2))


def extract_zip(zip_path: Path, extract_dir: Path):
    with zipfile.ZipFile(zip_path, "r") as zip_ref:
        zip_ref.extractall(extract_dir)


def main(zip_path: Path):
    extract_dir = zip_path.parent / zip_path.stem

    if zip_path.is_file() and zip_path.exists() and (not extract_dir.exists()):
        extract_zip(zip_path, extract_dir=extract_dir)
        final_extract_dir = extract_dir / extract_dir.name
    else:
        final_extract_dir = extract_dir / extract_dir.name

    image_dir = final_extract_dir / "corrected_images"

    assert (
        image_dir.exists() and image_dir.is_dir()
    ), f"Image directory not found: {image_dir}"

    image_paths = sorted(image_dir.glob("*.jpg"))

    omni_model = OmniNormalPredictor(device="cuda")
    dsine_model = DSineNormalPredictor(device="cuda")

    parent_path = Path("world")

    for idx, image_path in enumerate(image_paths):
        rr.set_time_sequence("timestep", idx)
        rgb = np.array(Image.open(image_path))

        omni_normal_b3hw = omni_model(rgb, None)
        dsine_normal_b3hw = dsine_model(rgb, None)

        # convert to numpy
        omni_normal_hw3 = omni_normal_b3hw[0].permute(1, 2, 0).numpy(force=True)
        dsine_normal_hw3 = dsine_normal_b3hw[0].permute(1, 2, 0).numpy(force=True)

        parent_path = Path("world")
        log_compare_normals(
            parent_path=parent_path,
            rgb=rgb,
            normal_np_1=omni_normal_hw3,
            normal_np_2=dsine_normal_hw3,
        )


if __name__ == "__main__":
    parser = ArgumentParser()
    parser.add_argument(
        "--zip-path",
        type=Path,
        required=True,
        help="Path to the image for inference",
    )
    rr.script_add_args(parser)

    args = parser.parse_args()
    rr.script_setup(args, "monopriors inference")
    main(args.zip_path)
    rr.script_teardown(args)



from jaxtyping import Float32, Int64, Bool
import numpy as np
from einops import rearrange


def estimate_intrinsics(
    H: int, W: int, fov: float = 55.0
) -> Float32[np.ndarray, "3 3"]:
    """
    Intrinsics for a pinhole camera model from image dimensions.
    Assume fov of 55 degrees and central principal point.
    """
    f = 0.5 * W / np.tan(0.5 * fov * np.pi / 180.0)
    cx = 0.5 * W
    cy = 0.5 * H
    K_33: Float32[np.ndarray, "3 3"] = np.array(
        [[f, 0, cx], [0, f, cy], [0, 0, 1]], dtype=np.float32
    )
    return K_33


def disparity_to_depth(
    disparity: Float32[np.ndarray, "h w"], focal_length: int, baseline: float = 1.0
) -> Float32[np.ndarray, "h w"]:
    range: float = float(np.minimum(disparity.max() / (disparity.min() + 1e-6), 100.0))
    disparity_max: float = float(disparity.max())
    min_disparity_range: float = disparity_max / range

    depth: Float32[np.ndarray, "h w"] = (focal_length * baseline) / np.maximum(
        disparity, min_disparity_range
    )
    # gamma correction for better visualizationg
    depth: Float32[np.ndarray, "h w"] = np.power(depth, 1.0 / 2.2)
    return depth


def depth_to_disparity(
    depth: Float32[np.ndarray, "h w"], focal_length: int, baseline: float = 1.0
) -> Float32[np.ndarray, "h w"]:
    disparity = (focal_length * baseline) / (depth + 0.01)
    return disparity


def depth_edges_mask(
    depth: Float32[np.ndarray, "h w"], threshold: float = 0.1
) -> Bool[np.ndarray, "h w"]:
    """Returns a mask of edges in the depth map.
    Args:
    depth: 2D numpy array of shape (H, W) with dtype float32.
    Returns:
    mask: 2D numpy array of shape (H, W) with dtype bool.
    """
    # Compute the x and y gradients of the depth map.
    depth_dx, depth_dy = np.gradient(depth)
    # Compute the gradient magnitude.
    depth_grad = np.sqrt(depth_dx**2 + depth_dy**2)
    # Compute the edge mask.
    mask: Bool[np.ndarray, "h w"] = depth_grad > threshold
    return mask


def depth_to_points(
    depth_1hw: Float32[np.ndarray, "1 h w"],
    K_33: Float32[np.ndarray, "3 3"],
    R=None,
    t=None,
) -> Float32[np.ndarray, "h w 3"]:
    K_33_inv: Float32[np.ndarray, "3 3"] = np.linalg.inv(K_33)
    if R is None:
        R: Float32[np.ndarray, "3 3"] = np.eye(3, dtype=np.float32)
    if t is None:
        t: Float32[np.ndarray, "3"] = np.zeros(3, dtype=np.float32)

    _, height, width = depth_1hw.shape

    # create 3d points grid
    x: Int64[np.ndarray, " h"] = np.arange(width)
    y: Int64[np.ndarray, " w"] = np.arange(height)
    coord: Int64[np.ndarray, "h w 2"] = np.stack(np.meshgrid(x, y), -1)
    z: Int64[np.ndarray, "h w 1"] = np.ones_like(coord)[:, :, [0]]
    coord = np.concatenate((coord, z), -1).astype(np.float32)  # z=1
    coord: Float32[np.ndarray, "1 h w 3"] = rearrange(coord, "h w c -> 1 h w c")

    # from depth to 3D points
    depth_1hw11: Float32[np.ndarray, "1 h w 1 1"] = rearrange(
        depth_1hw, "1 h w -> 1 h w 1 1"
    )

    # back project points from pixels to camera coordinate system
    pts3D_1 = (
        depth_1hw11
        * rearrange(K_33_inv, "h w -> 1 1 1 h w")
        @ rearrange(coord, "1 h w c -> 1 h w c 1")
    )

    # transform from camera to world coordinate system
    pts3D_2: Float32[np.ndarray, "1 h w 3 1"] = rearrange(
        R, "h w -> 1 1 1 h w"
    ) @ pts3D_1 + rearrange(t, "s -> 1 1 1 s 1")

    # rearrange to 3D points
    pointcloud: Float32[np.ndarray, "h w 3"] = rearrange(pts3D_2, "1 h w c 1 -> h w c")
    return pointcloud


def clip_disparity(disparity):
    # Get the 95th percentile of the disparity values
    p95 = np.percentile(disparity, 99)
    # Clip the disparity values at 105% of the 95th percentile
    clipped_disparity = np.clip(disparity, a_min=None, a_max=1.01 * p95)
    # normalize the disparity to 0-1
    clipped_disparity = (clipped_disparity - clipped_disparity.min()) / (
        clipped_disparity.max() - clipped_disparity.min()
    )
    clipped_disparity = (clipped_disparity * 255).astype(np.uint8)
    return clipped_disparity



from typing import Literal, TypedDict
import torch
import numpy as np
from jaxtyping import Float, UInt8
from timeit import default_timer as timer
from monopriors.depth_utils import estimate_intrinsics, disparity_to_depth
from transformers import pipeline
from PIL import Image
from einops import rearrange
from jaxtyping import Float32
from .base_relative_depth import RelativeDepthPrediction, BaseRelativePredictor


class DepthDict(TypedDict):
    predicted_depth: (
        Float32[torch.Tensor, "1 518 756"] | Float32[torch.Tensor, "1 756 518"]
    )
    depth: Image.Image


class DepthAnythingV1Predictor(BaseRelativePredictor):
    def __init__(
        self,
        device: Literal["cpu", "cuda"],
    ) -> None:
        super().__init__()
        print("Loading DepthAnythingV1 model...")
        start = timer()
        self.pipe = pipeline(
            task="depth-estimation",
            model="LiheYoung/depth-anything-small-hf",
            device=device,
        )
        print(f"DepthAnythingV1 model loaded. Time: {timer() - start:.2f}s")

    def __call__(
        self, rgb: UInt8[np.ndarray, "h w 3"], K_33: Float[np.ndarray, "3 3"] | None
    ) -> RelativeDepthPrediction:
        h, w, _ = rgb.shape
        # Transformers pipeline doesn't work with numpy/cv2, requires pil
        pil_img: Image.Image = Image.fromarray(rgb)
        depth_dict: DepthDict = self.pipe(pil_img)

        # depth is actually disparity here, interpolate to the original size
        disparity_bchw: Float32[torch.Tensor, "1 1 h w"] = (
            torch.nn.functional.interpolate(
                rearrange(depth_dict["predicted_depth"], "1 h w -> 1 1 h w"),
                (h, w),
                mode="bilinear",
            )
        )

        if K_33 is None:
            K_33 = estimate_intrinsics(H=h, W=w)

        disparity: Float32[np.ndarray, "h w"] = rearrange(
            disparity_bchw, "1 1 h w -> h w"
        ).numpy(force=True)

        relative_prediction = RelativeDepthPrediction(
            disparity=disparity,
            depth=disparity_to_depth(disparity, focal_length=int(K_33[0, 0])),
            confidence=np.ones_like(disparity),
            K_33=K_33,
        )

        return relative_prediction

    def set_model_device(self, device: Literal["cpu", "cuda"] = "cuda") -> None:
        # uses pipeline instead of model so not implemented
        self.pipe = pipeline(
            task="depth-estimation",
            model="LiheYoung/depth-anything-small-hf",
            device=device,
        )



from dataclasses import dataclass, field
from pathlib import Path
import json
import numpy as np
from jaxtyping import Float32
import calibur


@dataclass
class PolycamCameraData:
    blur_score: float
    cx: float
    cy: float
    fx: float
    fy: float
    height: int
    manual_keyframe: bool
    t_00: float
    t_01: float
    t_02: float
    t_03: float
    t_10: float
    t_11: float
    t_12: float
    t_13: float
    t_20: float
    t_21: float
    t_22: float
    t_23: float
    timestamp: int
    width: int
    neighbors: list[int] = field(default_factory=list)  # default value is an empty list

    def __post_init__(self) -> None:
        self.K_33: Float32[np.ndarray, "3 3"] = self.calculate_k33()
        self.world_T_cam_44: Float32[np.ndarray, "4 4"] = (
            self.calculate_world_t_cam_44()
        )
        self.cam_T_world_44: Float32[np.ndarray, "4 4"] = np.linalg.inv(
            self.world_T_cam_44
        )

    def calculate_k33(self) -> Float32[np.ndarray, "3 3"]:
        return np.array(
            [
                [self.fx, 0, self.cx],
                [0, self.fy, self.cy],
                [0, 0, 1],
            ],
            dtype=np.float32,
        )

    def calculate_world_t_cam_44(self) -> Float32[np.ndarray, "4 4"]:
        world_T_cam_44_gl = np.array(
            [
                [self.t_00, self.t_01, self.t_02, self.t_03],
                [self.t_10, self.t_11, self.t_12, self.t_13],
                [self.t_20, self.t_21, self.t_22, self.t_23],
                [0, 0, 0, 1],
            ],
            dtype=np.float32,
        )
        world_T_cam_44_cv = calibur.convert_pose(
            world_T_cam_44_gl,
            src_convention=calibur.CC.GL,
            dst_convention=calibur.CC.CV,
        )
        return world_T_cam_44_cv


def load_raw_polycam_data(camera_path: Path) -> PolycamCameraData:
    assert camera_path.name.endswith(".json"), "Camera parameters must be a JSON file"
    with open(camera_path, "r") as f:
        data = json.load(f)

    polycam_camera_data = PolycamCameraData(**data)
    return polycam_camera_data



from abc import ABC, abstractmethod
from dataclasses import dataclass
from jaxtyping import Float, UInt8
from typing import Literal
import numpy as np


@dataclass
class SurfaceNormalPrediction:
    normal_hw3: Float[np.ndarray, "h w 3"]
    # surface normal prediction
    confidence_hw1: Float[np.ndarray, "h w 1"]
    # confidence values


class BaseNormalPredictor(ABC):
    @abstractmethod
    def __call__(
        self, rgb: UInt8[np.ndarray, "h w 3"], K_33: Float[np.ndarray, "3 3"] | None
    ) -> SurfaceNormalPrediction:
        raise NotImplementedError

    def set_model_device(self, device: Literal["cpu", "cuda"] = "cuda") -> None:
        self.model.to(device)



import numpy as np
from pathlib import Path
import rerun as rr
import rerun.blueprint as rrb
from jaxtyping import UInt8, Float64, Float32, Bool
from monopriors.relative_depth_models import RelativeDepthPrediction, RELATIVE_PREDICTORS
from monopriors.metric_depth_models import MetricDepthPrediction, METRIC_PREDICTORS
from einops import rearrange
from monopriors.depth_utils import depth_to_points, clip_disparity, depth_edges_mask, depth_to_disparity


def log_relative_pred(
    parent_log_path: Path,
    relative_pred: RelativeDepthPrediction,
    rgb_hw3: UInt8[np.ndarray, "h w 3"],
    remove_flying_pixels: bool = True,
    jpeg_quality: int = 90,
    depth_edge_threshold: float = 1.1,
) -> None:
    cam_log_path: Path = parent_log_path / "camera"
    pinhole_path: Path = cam_log_path / "pinhole"

    # assume camera is at the origin
    cam_T_world_44: Float64[np.ndarray, "4 4"] = np.eye(4)

    rr.log(
        f"{cam_log_path}",
        rr.Transform3D(
            translation=cam_T_world_44[:3, 3],
            mat3x3=cam_T_world_44[:3, :3],
            from_parent=True,
        ),
    )
    rr.log(
        f"{pinhole_path}",
        rr.Pinhole(
            image_from_camera=relative_pred.K_33,
            width=rgb_hw3.shape[1],
            height=rgb_hw3.shape[0],
            camera_xyz=rr.ViewCoordinates.RDF,
        ),
    )
    rr.log(
        f"{pinhole_path}/image", rr.Image(rgb_hw3).compress(jpeg_quality=jpeg_quality)
    )

    depth_hw: Float32[np.ndarray, "h w"] = relative_pred.depth
    if remove_flying_pixels:
        edges_mask: Bool[np.ndarray, "h w"] = depth_edges_mask(
            depth_hw, threshold=depth_edge_threshold
        )
        depth_hw: Float32[np.ndarray, "h w"] = depth_hw * ~edges_mask

    rr.log(f"{pinhole_path}/depth", rr.DepthImage(depth_hw))

    # removes outliers from disparity (sometimes we can get weirdly large values)
    clipped_disparity: UInt8[np.ndarray, "h w"] = clip_disparity(
        relative_pred.disparity
    )

    # log to cam_log_path to avoid backprojecting disparity
    rr.log(f"{cam_log_path}/disparity", rr.DepthImage(clipped_disparity))

    depth_1hw: Float32[np.ndarray, "1 h w"] = rearrange(depth_hw, "h w -> 1 h w")
    pts_3d: Float32[np.ndarray, "h w 3"] = depth_to_points(
        depth_1hw, relative_pred.K_33
    )

    rr.log(
        f"{parent_log_path}/point_cloud",
        rr.Points3D(
            positions=pts_3d.reshape(-1, 3),
            colors=rgb_hw3.reshape(-1, 3),
        ),
    )


def create_compare_depth_blueprint(
    model_names: list[RELATIVE_PREDICTORS | METRIC_PREDICTORS],
) -> rrb.Blueprint:
    # model_names: list[str] = [model.__class__.__name__ for model in models]
    contents = [
        rrb.Spatial3DView(origin=f"{model_names[0]}"),
        rrb.Vertical(
            rrb.Spatial2DView(
                origin=f"{model_names[0]}/camera/pinhole/image",
            ),
            rrb.Spatial2DView(
                origin=f"{model_names[0]}/camera/pinhole/depth",
            ),
            rrb.Spatial2DView(
                origin=f"{model_names[0]}/camera/disparity",
            ),
        ),
        rrb.Spatial3DView(origin=f"{model_names[1]}"),
        rrb.Vertical(
            rrb.Spatial2DView(
                origin=f"{model_names[1]}/camera/pinhole/image",
            ),
            rrb.Spatial2DView(
                origin=f"{model_names[1]}/camera/pinhole/depth",
            ),
            rrb.Spatial2DView(
                origin=f"{model_names[1]}/camera/disparity",
            ),
        ),
    ]
    blueprint = rrb.Blueprint(
        rrb.Horizontal(
            contents=contents,
            column_shares=(3, 1, 3, 1),
        ),
        collapse_panels=True,
    )
    return blueprint


def log_metric_pred(parent_log_path: Path,
    metric_pred: MetricDepthPrediction,
    rgb_hw3: UInt8[np.ndarray, "h w 3"],
    remove_flying_pixels: bool = True,
    jpeg_quality: int = 90,
    depth_edge_threshold: float = 1.1,
) -> None:
    cam_log_path: Path = parent_log_path / "camera"
    pinhole_path: Path = cam_log_path / "pinhole"

    # assume camera is at the origin
    cam_T_world_44: Float64[np.ndarray, "4 4"] = np.eye(4)

    rr.log(
        f"{cam_log_path}",
        rr.Transform3D(
            translation=cam_T_world_44[:3, 3],
            mat3x3=cam_T_world_44[:3, :3],
            from_parent=True,
        ),
    )
    rr.log(
        f"{pinhole_path}",
        rr.Pinhole(
            image_from_camera=metric_pred.K_33,
            width=rgb_hw3.shape[1],
            height=rgb_hw3.shape[0],
            camera_xyz=rr.ViewCoordinates.RDF,
        ),
    )
    rr.log(
        f"{pinhole_path}/image", rr.Image(rgb_hw3).compress(jpeg_quality=jpeg_quality)
    )

    depth_hw: Float32[np.ndarray, "h w"] = metric_pred.depth_meters
    if remove_flying_pixels:
        edges_mask: Bool[np.ndarray, "h w"] = depth_edges_mask(
            depth_hw, threshold=depth_edge_threshold
        )
        depth_hw: Float32[np.ndarray, "h w"] = depth_hw * ~edges_mask

    rr.log(f"{pinhole_path}/depth", rr.DepthImage(depth_hw,meter=1.0))

    # removes outliers from disparity (sometimes we can get weirdly large values)
    clipped_disparity: Float32[np.ndarray, "h w"] = depth_to_disparity(
        depth_hw,
        focal_length=int(metric_pred.K_33[0, 0]),
        baseline=1000.0
        )

    # log to cam_log_path to avoid backprojecting disparity
    rr.log(f"{cam_log_path}/disparity", rr.DepthImage(clipped_disparity))

    depth_1hw: Float32[np.ndarray, "1 h w"] = rearrange(depth_hw, "h w -> 1 h w")
    pts_3d: Float32[np.ndarray, "h w 3"] = depth_to_points(
        depth_1hw, metric_pred.K_33
    )

    rr.log(
        f"{parent_log_path}/point_cloud",
        rr.Points3D(
            positions=pts_3d.reshape(-1, 3),
            colors=rgb_hw3.reshape(-1, 3),
        ),
    )


from pathlib import Path
from typing import Literal
import torch
import numpy as np
from jaxtyping import Float, UInt8
import torchvision.transforms.functional as TF
from omnidata_tools.torch.modules.midas.dpt_depth import DPTDepthModel
from monopriors.surface_normal_models.base_normal_model import (
    BaseNormalPredictor,
    SurfaceNormalPrediction,
)
from einops import rearrange


class OmniNormalPredictor(BaseNormalPredictor):
    def __init__(
        self, device: Literal["cpu", "cuda"], omnidata_pretrained_weights_path: Path
    ) -> None:
        self.device = device
        self.model = self._load_model(omnidata_pretrained_weights_path)
        self.image_size = 384

    def _load_model(self, omnidata_pretrained_weights_path: Path):
        model = DPTDepthModel(backbone="vitb_rn50_384", num_channels=3)  # DPT Hybrid
        omnidata_pretrained_weights_path = (
            omnidata_pretrained_weights_path / "omnidata_dpt_normal_v2.ckpt"
        )
        assert omnidata_pretrained_weights_path.exists(), "Weights not found"
        map_location = (
            (lambda storage, loc: storage.cuda())
            if torch.cuda.is_available()
            else torch.device("cpu")
        )
        checkpoint = torch.load(
            omnidata_pretrained_weights_path, map_location=map_location
        )

        if "state_dict" in checkpoint:
            state_dict = {}
            for k, v in checkpoint["state_dict"].items():
                state_dict[k[6:]] = v
        else:
            state_dict = checkpoint

        model.load_state_dict(state_dict)
        model.to(self.device)
        return model

    def __call__(
        self, rgb: UInt8[np.ndarray, "h w 3"], K_33: Float[np.ndarray, "3 3"] | None
    ) -> SurfaceNormalPrediction:
        rgb = rgb.astype(np.float32) / 255.0
        img_3hw = torch.from_numpy(rgb).permute(2, 0, 1)
        _, H, W = img_3hw.shape
        # omnidata normal model expects 384x384 images
        img_3hw = TF.resize(img_3hw, (self.image_size, self.image_size), antialias=None)
        img_b3hw = img_3hw.unsqueeze(0).to(self.device)

        # return normal_b3hw
        normal_b3hw = self.model(img_b3hw).clamp(min=0, max=1)
        normal_3hw = rearrange(normal_b3hw, "1 c h w -> c h w")
        # normal_3hw = normal_b3hw.squeeze(0)
        # reshape back to original size
        if normal_3hw.shape[1] != H and normal_3hw.shape[2] != W:
            normal_3hw = TF.resize(normal_3hw, (H, W), antialias=None)

        # generates normal that is between 0-1 in the OpenCV Format (RDF)
        # no confidence map is returned from omnidata model
        normal_pred = SurfaceNormalPrediction(
            normal_hw3=rearrange(normal_3hw, "c h w -> h w c"),
            confidence_hw1=np.ones((H, W, 1)),
        )
        return normal_pred



import gradio as gr
import rerun as rr
import tempfile

import numpy as np
from pathlib import Path
import os
from gradio_rerun import Rerun

from monopriors.relative_depth_models import (
    get_relative_predictor,
    BaseRelativePredictor,
    RelativeDepthPrediction,
    RELATIVE_PREDICTORS,
)
from monopriors.rr_logging_utils import log_relative_pred
from jaxtyping import UInt8
from typing import get_args
import gc
import torch
import mmcv

try:
    import spaces  # type: ignore
    IN_SPACES = True
except ImportError:
    print("Not running on Zero")
    IN_SPACES = False


MODELS_TO_SKIP: list[str] = []
model_load_status: str = "Models loaded and ready to use!"
if gr.NO_RELOAD:
    DEPTH_PREDICTOR: BaseRelativePredictor = get_relative_predictor(
        "DepthAnythingV2Predictor"
    )(device="cuda")


def predict_depth(
    rgb_hw3: UInt8[np.ndarray, "h w 3"]
) -> RelativeDepthPrediction:
    relative_pred: RelativeDepthPrediction = DEPTH_PREDICTOR.__call__(
            rgb=rgb_hw3, K_33=None
        )
    return relative_pred
    
if IN_SPACES:
    predict_depth = spaces.GPU(predict_depth)
    # remove any model that fails on zerogpu spaces
    MODELS_TO_SKIP.extend(["Metric3DRelativePredictor", "Metric3DPredictor"])

def load_model(
    model: RELATIVE_PREDICTORS,
    progress=gr.Progress(),
) -> str:
    print(model)
    # check if the models are in the list of models to skip
    if any(model == m for m in MODELS_TO_SKIP):
        raise gr.Error(
            f"Model not supported on ZeroGPU, please try another model: {MODELS_TO_SKIP}"
        )

    global DEPTH_PREDICTOR
    # delete the previous models and clear gpu memory
    if "DEPTH_PREDICTOR" in globals():
        del DEPTH_PREDICTOR

    gc.collect()
    torch.cuda.empty_cache()

    progress(0, desc="Loading Model please wait...")

    DEPTH_PREDICTOR = get_relative_predictor(model)(device="cuda")

    return model_load_status


@rr.thread_local_stream("depth_inference")
def relative_depth_from_img(
    rgb_hw3: UInt8[np.ndarray, "h w 3"],
    remove_flying_pixels: bool,
    depth_map_threshold: float,
    pending_cleanup: list[str],
) -> str:
    try:
        parent_log_path = Path("world")

        # resize the image to have a max dim of 1024
        max_dim:int = 1024
        height, width, _ = rgb_hw3.shape
        current_dim = max(height, width)
        if current_dim > max_dim:
            scale_factor = max_dim / current_dim
            rgb_hw3 = mmcv.imrescale(img=rgb_hw3, scale=scale_factor)

        relative_pred: RelativeDepthPrediction = predict_depth(rgb_hw3)
        rr.log(f"{parent_log_path}", rr.ViewCoordinates.RDF, timeless=True)
        log_relative_pred(
            parent_log_path,
            relative_pred,
            rgb_hw3,
            remove_flying_pixels=remove_flying_pixels,
            depth_edge_threshold=depth_map_threshold,
        )

        # We eventually want to clean up the RRD file after it's sent to the viewer, so tracking
        # any pending files to be cleaned up when the state is deleted.
        temp = tempfile.NamedTemporaryFile(prefix="depth_inf_", suffix=".rrd", delete=False)
        pending_cleanup.append(temp.name)

        rr.save(temp.name)
        return temp.name
    except NameError as e:
        raise gr.Error(f"Please wait Model is being loaded: {e}")
    except Exception as e:
        raise gr.Error(f"Error predicting depth: {e}")


def cleanup_rrds(pending_cleanup: list[str]) -> None:
    for f in pending_cleanup:
        os.unlink(f)


with gr.Blocks() as depth_inference_block:
    pending_cleanup = gr.State([], time_to_live=10, delete_callback=cleanup_rrds)
    with gr.Row():
        input_image = gr.Image(
            label="Input Image",
            image_mode="RGB",
            type="numpy",
            height=300,
        )
        with gr.Column():
            with gr.Row():
                remove_flying_pixels = gr.Checkbox(
                    label="Remove Flying Pixels",
                    value=True,
                    interactive=True,
                )
                depth_map_threshold = gr.Slider(
                    label="⬇️ number == more pruning ⬆️ less pruning",
                    minimum=0.05,
                    maximum=0.95,
                    step=0.05,
                    value=0.1,
                )
            with gr.Row():
                model_dropdown = gr.Dropdown(
                    choices=list(get_args(RELATIVE_PREDICTORS)),
                    label="Model",
                    value="DepthAnythingV2Predictor",
                    interactive=True,
                )
            with gr.Row():
                model_type = gr.Radio(
                    choices=["Metric (TODO)", "Relative"],
                    value="Relative",
                )
                model_status = gr.Textbox(
                    label="Model Status",
                    value=model_load_status,
                    interactive=False,
                )
    with gr.Row():
        run_btn = gr.Button("Run Depth Inference")
        load_model_btn = gr.Button("Load Model")
    with gr.Row():
        rr_viewer = Rerun(
            streaming=True,
            panel_states={
                "time": "collapsed",
                "blueprint": "collapsed",
                "selection": "collapsed",
                "top": "collapsed",
            },
        )

    # get all jpegs in examples path
    examples_paths = Path("examples").glob("*.jpeg")
    # set the examples to be the sorted list of input parameterss (path, remove_flying_pixels, depth_map_threshold)
    examples_list = sorted([[str(path)] for path in examples_paths])
    examples = gr.Examples(
        examples=examples_list,
        inputs=[
            input_image,
            remove_flying_pixels,
            depth_map_threshold,
            pending_cleanup,
        ],
        outputs=[rr_viewer],
        fn=relative_depth_from_img,
        cache_examples=False,
    )

    run_btn.click(
        relative_depth_from_img,
        inputs=[
            input_image,
            remove_flying_pixels,
            depth_map_threshold,
            pending_cleanup,
        ],
        outputs=[rr_viewer],
    )


    load_model_btn.click(
        load_model,
        inputs=[model_dropdown],
        outputs=[model_status],
    )



from dataclasses import dataclass
from typing import List, Optional
import json
from pathlib import Path


@dataclass
class SceneBox:
    aabb: List[List[float]]
    near: float
    far: float
    radius: float
    collider_type: str


@dataclass
class SDFStudioFrame:
    # Assuming basic structure for a frame; add or remove fields as necessary
    rgb_path: Optional[str] = None
    camtoworld: Optional[List[List[float]]] = None
    intrinsics: Optional[List[List[float]]] = None
    mono_depth_path: Optional[str] = None
    mono_normal_path: Optional[str] = None
    foreground_mask: Optional[str] = None
    sfm_sparse_points_view: Optional[str] = None


@dataclass
class SDFStudioData:
    camera_model: str
    height: int
    width: int
    has_mono_prior: bool
    pairs: Optional[str]
    worldtogt: List[List[float]]
    scene_box: SceneBox
    frames: List[SDFStudioFrame]


def load_sdfstudio_from_json(json_path: Path) -> SDFStudioData:
    # load the meta.json file
    with open(json_path, "r") as f:
        data = json.load(f)

    scene_box_data = data.pop(
        "scene_box", {}
    )  # Use pop to remove 'scene_box' from 'data'
    scene_box = SceneBox(**scene_box_data) if scene_box_data else None

    frames_data = data.pop("frames", [])  # Similarly, remove 'frames' from 'data'
    frames = (
        [SDFStudioFrame(**frame_data) for frame_data in frames_data]
        if frames_data
        else []
    )

    # Filter out any unexpected keys from the data dictionary
    expected_keys = {
        "camera_model",
        "height",
        "width",
        "has_mono_prior",
        "pairs",
        "worldtogt",
    }
    filtered_data = {k: v for k, v in data.items() if k in expected_keys}

    return SDFStudioData(scene_box=scene_box, frames=frames, **filtered_data)



from abc import ABC, abstractmethod
from dataclasses import dataclass
from jaxtyping import Float32, UInt8
from typing import Literal
import numpy as np


@dataclass
class RelativeDepthPrediction:
    disparity: Float32[np.ndarray, "h w"]
    # relative disparity
    depth: Float32[np.ndarray, "h w"]
    # relative depth
    confidence: Float32[np.ndarray, "h w"]
    # confidence values
    K_33: Float32[np.ndarray, "3 3"]
    # intrinsics


class BaseRelativePredictor(ABC):
    @abstractmethod
    def __call__(
        self, rgb: UInt8[np.ndarray, "h w 3"], K_33: Float32[np.ndarray, "3 3"] | None
    ) -> RelativeDepthPrediction:
        raise NotImplementedError

    def set_model_device(self, device: Literal["cpu", "cuda"] = "cuda") -> None:
        self.model.to(device)



import os
from typing import Literal
import torch
import numpy as np
from jaxtyping import Float, UInt8
from torchvision import transforms
import torch.nn.functional as F
from monopriors.third_party.dsine.dsine import DSINE
from monopriors.third_party.dsine.dsine_kappa import DSINE_v02_kappa
from monopriors.third_party.dsine.utils.utils import get_intrins_from_fov, pad_input
from monopriors.surface_normal_models.base_normal_model import (
    BaseNormalPredictor,
    SurfaceNormalPrediction,
)
from einops import rearrange


class DSineNormalPredictor(BaseNormalPredictor):
    def __init__(
        self,
        device: Literal["cpu", "cuda"],
        model_type: Literal["dsine", "dsine_kappa"] = "dsine_kappa",
    ):
        self.device = device
        self.transform = transforms.Normalize(
            mean=[0.485, 0.456, 0.406], std=[0.229, 0.224, 0.225]
        )
        self.model = self.load_model(model_type=model_type)

    def load_model(self, model_type: Literal["dsine", "dsine_kappa"]):
        state_dict = self._load_state_dict(
            local_file_path="checkpoints/dsine_kappa.pt", model_type=model_type
        )
        if model_type == "dsine":
            model = DSINE()
        else:
            model = DSINE_v02_kappa()

        model.load_state_dict(state_dict, strict=True)
        model.eval()
        model = model.to(self.device)
        model.pixel_coords = model.pixel_coords.to(self.device)

        return model

    def _load_state_dict(
        self,
        local_file_path: str | None = None,
        model_type: Literal["dsine", "dsine_kappa"] = "dsine_kappa",
    ):
        if local_file_path is not None and os.path.exists(local_file_path):
            # Load state_dict from local file
            state_dict = torch.load(local_file_path, map_location=torch.device("cpu"))
        else:
            # Load state_dict from the default URL
            file_name = "dsine.pt" if model_type == "dsine" else "dsine_kappa.pt"
            url = (
                f"https://huggingface.co/camenduru/DSINE/resolve/main/dsine.pt"
                if model_type == "dsine"
                else f"https://huggingface.co/pablovela5620/dsine_kappa/resolve/main/dsine_kappa.pt"
            )
            state_dict = torch.hub.load_state_dict_from_url(
                url, file_name=file_name, map_location=torch.device("cpu")
            )

        return state_dict["model"]

    def __call__(
        self, rgb: UInt8[np.ndarray, "h w 3"], K_33: Float[np.ndarray, "3 3"] | None
    ) -> SurfaceNormalPrediction:
        # preprocess the input image
        rgb: Float[np.ndarray, "h w 3"] = rgb.astype(np.float32) / 255.0
        rgb: Float[torch.Tensor, "1 c h w"] = torch.from_numpy(
            rearrange(rgb, "h w c -> 1 c h w")
        ).to(self.device)

        _, _, h, w = rgb.shape

        # zero-pad the input image so that both the width and height are multiples of 32
        left, right, top, bottom = pad_input(h, w)
        rgb = F.pad(rgb, (left, right, top, bottom), mode="constant", value=0.0)
        rgb = self.transform(rgb)

        if K_33 is None:
            K_33: Float[torch.Tensor, "3 3"] = get_intrins_from_fov(
                new_fov=60.0, H=h, W=w, device=self.device
            )
            K_b33: Float[torch.Tensor, "b 3 3"] = rearrange(K_33, "r c -> 1 r c")
        else:
            K_b33 = torch.from_numpy(rearrange(K_33, "r c -> 1 r c")).to(self.device)

        # add padding to intrinsics
        K_b33[:, 0, 2] += left
        K_b33[:, 1, 2] += top

        with torch.no_grad():
            normal_list: list[torch.Tensor] = self.model(rgb, intrins=K_b33)
            # last value in the list is the normal map
            normal_bchw: (
                Float[torch.Tensor, "b 3 _ _"] | Float[torch.Tensor, "b 4 _ _"]
            ) = normal_list[-1]
            # undo padding
            normal_bchw: (
                Float[torch.Tensor, "b 3 _ _"] | Float[torch.Tensor, "b 4 _ _"]
            ) = normal_bchw[:, :, top : top + h, left : left + w]

            normal_b3hw: Float[torch.Tensor, "b 3 h w"]
            conf_b1hw: Float[torch.Tensor, "b 1 h w"] | None

            normal_b3hw, conf_b1hw = normal_bchw[:, :3, :, :], normal_bchw[:, 3:, :, :]
            conf_b1hw = None if conf_b1hw.size(1) == 0 else conf_b1hw

        normal_pred = SurfaceNormalPrediction(
            normal_hw3=rearrange(normal_b3hw, "1 c h w -> h w c").numpy(force=True),
            confidence_hw1=rearrange(conf_b1hw, "1 c h w -> h w c").numpy(force=True),
        )

        return normal_pred



from .base_metric_depth import MetricDepthPrediction, BaseMetricPredictor
from .unidepth import UniDepthMetricPredictor
from .metric3d import Metric3DPredictor
from typing import Literal, get_args, Callable

# Define predictor names as a list of strings
METRIC_PREDICTORS = Literal["UniDepthMetricPredictor", "Metric3DPredictor"]

# Use the list to generate the __all__ list
__all__: list[str] = list(get_args(METRIC_PREDICTORS)) + [
    "MetricDepthPrediction",
]


def get_metric_predictor(
    predictor_type: METRIC_PREDICTORS,
) -> Callable[..., BaseMetricPredictor]:
    match predictor_type:
        case "UniDepthMetricPredictor":
            return UniDepthMetricPredictor
        case "Metric3DPredictor":
            return Metric3DPredictor
        case _:
            raise ValueError(f"Unknown predictor type: {predictor_type}")



from abc import ABC, abstractmethod
from typing import Literal
import torch
import numpy as np
from dataclasses import dataclass
from jaxtyping import Float, UInt8
from monopriors.surface_normal_models import (
    get_normal_predictor,
    SurfaceNormalPrediction,
)
from monopriors.metric_depth_models import get_metric_predictor, MetricDepthPrediction
from einops import rearrange


@dataclass
class OldMonoPriorPrediction:
    depth_b1hw: Float[torch.Tensor, "b 1 h w"]
    normal_b3hw: Float[torch.Tensor, "b 3 h w"]
    K_b33: Float[torch.Tensor, "b 3 3"] | None = None
    depth_conf_b1hw: Float[torch.Tensor, "b 1 h w"] | None = None
    normal_conf_b1hw: Float[torch.Tensor, "b 1 h w"] | None = None

    def to_numpy(
        self,
    ) -> tuple[
        Float[np.ndarray, "b h w 1"],
        Float[np.ndarray, "b h w 3"],
        Float[np.ndarray, "b 3 3"] | None,
        Float[np.ndarray, "b h w 1"] | None,
        Float[np.ndarray, "b h w 1"] | None,
    ]:
        depth_np_bhw1 = rearrange(self.depth_b1hw, "b c h w -> b h w c").numpy(
            force=True
        )
        normal_np_bhw3 = rearrange(self.normal_b3hw, "b c h w -> b h w c").numpy(
            force=True
        )
        K_np_b33 = self.K_b33.numpy(force=True) if self.K_b33 is not None else None
        depth_conf_np_bhw1 = (
            rearrange(self.depth_conf_b1hw, "b c h w -> b h w c").numpy(force=True)
            if self.depth_conf_b1hw is not None
            else None
        )
        normal_conf_np_bhw1 = (
            rearrange(self.normal_conf_b1hw, "b c h w -> b h w c").numpy(force=True)
            if self.normal_conf_b1hw is not None
            else None
        )
        return (
            depth_np_bhw1,
            normal_np_bhw3,
            K_np_b33,
            depth_conf_np_bhw1,
            normal_conf_np_bhw1,
        )


@dataclass
class MonoPriorPrediction:
    metric_pred: MetricDepthPrediction
    normal_pred: SurfaceNormalPrediction


class MonoPriorModel(ABC):
    def __init__(self) -> None:
        self.device: Literal["cuda", "cpu"] = (
            "cuda" if torch.cuda.is_available() else "cpu"
        )

    @abstractmethod
    def __call__(
        self, rgb: UInt8[np.ndarray, "h w 3"], K_33: Float[np.ndarray, "3 3"] | None
    ) -> MonoPriorPrediction:
        raise NotImplementedError


class DsineAndUnidepth(MonoPriorModel):
    def __init__(self) -> None:
        super().__init__()
        self.depth_model = get_metric_predictor("UniDepthMetricPredictor")(
            device=self.device
        )
        self.surface_model = get_normal_predictor("DSineNormalPredictor")(
            device=self.device
        )

    def __call__(
        self,
        rgb: UInt8[np.ndarray, "h w 3"],
        K_33: Float[np.ndarray, "3 3"] | None = None,
    ) -> MonoPriorPrediction:
        metric_pred: MetricDepthPrediction = self.depth_model.__call__(rgb, K_33)
        normal_pred: SurfaceNormalPrediction = self.surface_model(rgb, K_33)

        return MonoPriorPrediction(metric_pred=metric_pred, normal_pred=normal_pred)



import numpy as np
import rerun as rr
from pathlib import Path
from argparse import ArgumentParser
import cv2

from jaxtyping import UInt8, Float32
from monopriors.data.sdfstudio_data import load_sdfstudio_from_json


def main(data_dir: Path):
    assert data_dir.is_dir(), f"{data_dir} is not a directory"
    assert data_dir.exists(), f"{data_dir} does not exist"
    meta_json_path = data_dir / "meta_data.json"
    assert meta_json_path.exists(), f"{meta_json_path} does not exist"

    data = load_sdfstudio_from_json(meta_json_path)

    parent_log_path = Path("world")

    for idx, frame in enumerate(data.frames):
        cam_path = parent_log_path / f"cam_{idx}"
        pinhole_path = cam_path / "pinhole"
        rgb_path = data_dir / frame.rgb_path
        depth_path = data_dir / frame.mono_depth_path
        normal_path = data_dir / frame.mono_normal_path

        rgb: UInt8[np.ndarray, "h w 3"] = cv2.cvtColor(
            cv2.imread(str(rgb_path)), cv2.COLOR_BGR2RGB
        )
        depth: Float32[np.ndarray, "h w"] = np.load(depth_path)
        # between 0 and 1
        normal: Float32[np.ndarray, "3 h w"] = np.load(normal_path)
        # permute the normal to be in the correct format
        normal = np.transpose(normal, (1, 2, 0))
        # between -1 and 1
        normal = (
            normal * 2.0 - 1.0
        )  # omnidata output is normalized so we convert it back to normal here
        # change coordinate system
        normal[..., 2] *= -1

        world_T_cam_44 = np.array(frame.camtoworld)
        K_44 = np.array(frame.intrinsics)
        rr.log(f"{pinhole_path}/image", rr.Image(rgb))
        rr.log(f"{pinhole_path}/depth", rr.DepthImage(depth))
        rr.log(f"{pinhole_path}/normal", rr.Image(normal))
        break


if __name__ == "__main__":
    parser = ArgumentParser()
    parser.add_argument(
        "--sdfstudio-data-dir",
        type=Path,
        default="data/sdfstudio/sdfstudio/replica/scan5",
    )
    rr.script_add_args(parser)
    args = parser.parse_args()
    rr.script_setup(args, "sdfstudio-viewer")
    main(args.sdfstudio_data_dir)
    rr.script_teardown(args)



from typing import Literal
import torch
import numpy as np
from jaxtyping import Float, UInt8, Float32
from timeit import default_timer as timer
from monopriors.depth_utils import estimate_intrinsics, depth_to_disparity
from monopriors.relative_depth_models.base_relative_depth import (
    RelativeDepthPrediction,
    BaseRelativePredictor,
)
from einops import rearrange
import cv2
from typing import TypedDict


class Metric3DPredDict(TypedDict):
    prediction: torch.Tensor
    predictions_list: list[torch.Tensor]
    confidence: torch.Tensor
    confidence_list: list[torch.Tensor]
    pred_logit: torch.Tensor
    prediction_normal: torch.Tensor
    normal_out_list: list[torch.Tensor]
    low_resolution_init: torch.Tensor


class Metric3DRelativePredictor(BaseRelativePredictor):
    def __init__(
        self,
        device: Literal["cpu", "cuda"],
    ):
        super().__init__()
        self.device = device
        print("Loading Relative Metric3D model...")
        start = timer()
        self.model = (
            torch.hub.load("yvanyin/metric3d", "metric3d_vit_small", pretrain=True)
            .to(device)
            .eval()
        )
        print(f"Metric3D model loaded. Time: {timer() - start:.2f}s")

        # only for VIT models
        self.input_height: int = 616
        self.input_width: int = 1064

        self.padding_value: list[float] = [123.675, 116.28, 103.53]

    def __call__(
        self,
        rgb: UInt8[np.ndarray, "h w 3"],
        K_33: Float[np.ndarray, "3 3"] | None,
    ) -> RelativeDepthPrediction:
        h, w, _ = rgb.shape
        scale: float = min(self.input_height / h, self.input_width / w)
        rgb_rescaled: UInt8[np.ndarray, "_ _ 3"] = cv2.resize(
            rgb, (int(w * scale), int(h * scale)), interpolation=cv2.INTER_LINEAR
        )

        k33_rescaled: Float[np.ndarray, "3 3"] | None = (
            K_33 * scale if K_33 is not None else None
        )

        rgb_padded: UInt8[np.ndarray, "616 1064 3"]
        pad_info: list[int]
        rgb_padded, pad_info = self.pad_rgb(rgb_rescaled)

        ##### normalize
        mean: Float32[torch.Tensor, "c 1 1"] = rearrange(
            torch.tensor([123.675, 116.28, 103.53]), "c -> c 1 1"
        )
        std: Float32[torch.Tensor, "c 1 1"] = rearrange(
            torch.tensor([58.395, 57.12, 57.375]), "c -> c 1 1"
        )
        rgb_tensor_hwc: UInt8[torch.Tensor, "616 1064 c"] = torch.from_numpy(rgb_padded)
        rgb_tensor_bchw: Float32[torch.Tensor, "b c 616 1064"] = rearrange(
            rgb_tensor_hwc.float(), "h w c -> 1 c h w"
        )
        rgb_tensor_bchw: Float32[torch.Tensor, "b c 616 1064"] = (
            rgb_tensor_bchw - mean
        ) / std
        rgb_tensor_bchw: Float32[torch.Tensor, "b c 616 1064"] = rgb_tensor_bchw.to(
            self.device
        )

        with torch.no_grad():
            pred_depth_b1hw: Float32[torch.Tensor, "b 1 616 1064"]
            confidence_b1hw: Float32[torch.Tensor, "b 1 616 1064"]
            output_dict: Metric3DPredDict
            pred_depth_b1hw, confidence_b1hw, output_dict = self.model.inference(
                {"input": rgb_tensor_bchw}
            )

        # un pad
        pred_depth_hw: Float32[torch.Tensor, "616 1064"] = rearrange(
            pred_depth_b1hw, "1 1 h w -> h w"
        )
        pred_depth_hw: Float32[torch.Tensor, "_ _"] = pred_depth_hw[
            pad_info[0] : pred_depth_hw.shape[0] - pad_info[1],
            pad_info[2] : pred_depth_hw.shape[1] - pad_info[3],
        ]

        # # upsample to original size
        pred_depth_bchw = torch.nn.functional.interpolate(
            rearrange(pred_depth_hw, "h w -> 1 1 h w"), (h, w), mode="bilinear"
        )
        pred_depth_hw = rearrange(pred_depth_bchw, "1 1 h w -> h w")

        # confidence_b1hw: Float32[torch.Tensor, "b 1 h w"] = (
        #     torch.nn.functional.interpolate(confidence_b1hw, (h, w), mode="bilinear")
        # )
        # conf_tensor_hw: Float32[torch.Tensor, "h w"] = rearrange(
        #     confidence_b1hw, "1 1 h w -> h w"
        # )
        # conf_hw: Float32[np.ndarray, "h w"] = conf_tensor_hw.numpy(force=True)

        # #### de-canonical transform
        # 1000.0 is the focal length of canonical camera
        if k33_rescaled is not None:
            canonical_to_real_scale = k33_rescaled[0, 0] / 1000.0
            # now the depth is metric
            pred_depth_metric_hw = pred_depth_hw * canonical_to_real_scale
            pred_depth_metric_hw = torch.clamp(pred_depth_metric_hw, 0, 300)
        else:
            K_33 = estimate_intrinsics(rgb.shape[0], rgb.shape[1])

        pred_depth_hw = pred_depth_hw.numpy(force=True)
        disparity = depth_to_disparity(pred_depth_hw, focal_length=int(K_33[0, 0]))

        relative_pred = RelativeDepthPrediction(
            disparity=disparity,
            depth=pred_depth_hw,
            # TODO add confidence
            confidence=np.ones_like(pred_depth_hw),
            K_33=K_33,
        )

        return relative_pred

    def pad_rgb(
        self, rgb_resized: UInt8[np.ndarray, "_ _ 3"]
    ) -> tuple[UInt8[np.ndarray, "616 1064 3"], list[int]]:
        h_new, w_new, _ = rgb_resized.shape
        pad_h = self.input_height - h_new
        pad_w = self.input_width - w_new
        pad_h_half = pad_h // 2
        pad_w_half = pad_w // 2
        rgb_padded: UInt8[np.ndarray, "616 1064 3"] = cv2.copyMakeBorder(
            rgb_resized,
            pad_h_half,
            pad_h - pad_h_half,
            pad_w_half,
            pad_w - pad_w_half,
            cv2.BORDER_CONSTANT,
            value=self.padding_value,
        )
        pad_info = [pad_h_half, pad_h - pad_h_half, pad_w_half, pad_w - pad_w_half]
        return rgb_padded, pad_info



from typing import Literal
import torch
import numpy as np
from jaxtyping import Float, UInt8
from timeit import default_timer as timer
from monopriors.metric_depth_models.base_metric_depth import (
    MetricDepthPrediction,
    BaseMetricPredictor,
)
from einops import rearrange


class UniDepthMetricPredictor(BaseMetricPredictor):
    def __init__(
        self,
        device: Literal["cpu", "cuda"],
        version: Literal["v1", "v2"] = "v2",
        backbone: Literal["vits14", "vitl14"] = "vitl14",
    ):
        super().__init__()
        print("Loading UniDepth model...")
        start = timer()
        self.model = (
            torch.hub.load(
                "lpiccinelli-eth/UniDepth",
                "UniDepth",
                version=version,
                backbone=backbone,
                pretrained=True,
                trust_repo=True,
            )
            .to(device)
            .eval()
        )
        print(f"UniDepth model loaded. Time: {timer() - start:.2f}s")

    def __call__(
        self,
        rgb: UInt8[np.ndarray, "h w 3"],
        K_33: Float[np.ndarray, "3 3"] | None,
    ) -> MetricDepthPrediction:
        # Load the RGB image and the normalization will be taken care of by the model
        # rgb = torch.from_numpy(rgb).permute(2, 0, 1)  # C, H, W
        rgb = rearrange(rgb, "h w c -> c h w")
        rgb = torch.from_numpy(rgb)

        if K_33 is None:
            predictions = self.model.infer(rgb)
        else:
            K_33 = torch.from_numpy(K_33)
            predictions = self.model.infer(rgb, K_33)

        depth_b1hw: Float[torch.Tensor, "b 1 h w"] = predictions["depth"]
        K_b33: Float[torch.Tensor, "b 3 3"] = predictions["intrinsics"]
        conf_b1hw: Float[torch.Tensor, "b 1 h w"] = predictions["confidence"]

        assert depth_b1hw.shape[0] == 1, "Batch size must be 1"

        # normalize the confidence to 0-1
        conf_b1hw = (conf_b1hw - conf_b1hw.min()) / (conf_b1hw.max() - conf_b1hw.min())

        # convert to numpy and rearrange
        depth_hw = rearrange(depth_b1hw, "1 1 h w -> h w").numpy(force=True)
        conf_hw = rearrange(conf_b1hw, "1 1 h w -> h w").numpy(force=True)
        # rearrange doesn't work here?
        K_33 = K_b33.squeeze(0).numpy(force=True)

        metric_pred = MetricDepthPrediction(
            depth_meters=depth_hw,
            confidence=conf_hw,
            K_33=K_33,
        )

        return metric_pred



from argparse import ArgumentParser, Namespace
from typing import TypedDict
import os
from huggingface_hub import upload_file, upload_folder
from tqdm import tqdm
from pathlib import Path


class FileUpload(TypedDict):
    local_path: str
    repo_path: str


def main(upload_examples: bool) -> None:
    space_id: str | None = os.environ.get("SPACE_ID")
    whl_path: str | None = os.environ.get("WHL_PATH")

    assert space_id is not None, "Please set the SPACE_ID environment variable"
    assert whl_path is not None, "Please set the WHL_PATH environment variable"

    print(f"SPACE_ID: {space_id}")
    print(f"WHL_PATH: {whl_path}")

    files_to_upload: list[FileUpload] = [
        {"local_path": "./tools/app.py", "repo_path": "app.py"},
        {"local_path": "./tools/gradio_app.py", "repo_path": "gradio_app.py"},
        {
            "local_path": f"./dist/{whl_path}",
            "repo_path": f"{whl_path}",
        },
        {"local_path": "./pyproject.toml", "repo_path": "pyproject.toml"},
        {"local_path": "./pixi.lock", "repo_path": "pixi.lock"},
        {"local_path": "./.pixi.sh", "repo_path": ".pixi.sh"},
    ]

    for file in tqdm(files_to_upload, desc="Uploading files"):
        assert os.path.exists(
            file["local_path"]
        ), f"File {file['local_path']} does not exist"
        with open(file["local_path"], "rb") as fobj:
            upload_file(
                path_or_fileobj=fobj,
                path_in_repo=file["repo_path"],
                repo_id=space_id,
                repo_type="space",
            )

    if upload_examples:
        examples_path = Path("examples")
        assert examples_path.exists(), f"Examples folder {examples_path} does not exist"
        upload_folder(
            folder_path=str(examples_path),
            path_in_repo=str(examples_path),
            repo_id=space_id,
            repo_type="space",
        )


if __name__ == "__main__":
    parser = ArgumentParser()
    parser.add_argument("--upload-examples", action="store_true")
    args: Namespace = parser.parse_args()
    main(upload_examples=args.upload_examples)



from dataclasses import dataclass
from typing import List
import json
from pathlib import Path


@dataclass
class NerfStudioFrame:
    # Assuming basic structure for a frame; add or remove fields as necessary
    fl_x: float
    fl_y: float
    cx: float
    cy: float
    w: int
    h: int
    file_path: str
    transform_matrix: List[List[float]]


@dataclass
class NerfStudioData:
    camera_model: str
    orientation_override: str
    frames: List[NerfStudioFrame]


def load_nerfstudio_from_json(json_path: Path) -> NerfStudioData:
    # load the meta.json file
    with open(json_path, "r") as f:
        data = json.load(f)

    frames_data = data.pop("frames", [])  # Use pop to remove 'scene_box' from 'data'
    frames = [NerfStudioFrame(**frame_data) for frame_data in frames_data]

    # Filter out any unexpected keys from the data dictionary
    expected_keys = {
        "camera_model",
        "orientation_override",
    }
    filtered_data = {k: v for k, v in data.items() if k in expected_keys}

    return NerfStudioData(frames=frames, **filtered_data)



"""utils"""

import os
import torch
import numpy as np


def load_checkpoint(fpath, model):
    print("loading checkpoint... {}".format(fpath))

    ckpt = torch.load(fpath, map_location="cpu")["model"]

    load_dict = {}
    for k, v in ckpt.items():
        if k.startswith("module."):
            k_ = k.replace("module.", "")
            load_dict[k_] = v
        else:
            load_dict[k] = v

    model.load_state_dict(load_dict)
    print("loading checkpoint... / done")
    return model


def compute_normal_error(pred_norm, gt_norm):
    pred_error = torch.cosine_similarity(pred_norm, gt_norm, dim=1)
    pred_error = torch.clamp(pred_error, min=-1.0, max=1.0)
    pred_error = torch.acos(pred_error) * 180.0 / np.pi
    pred_error = pred_error.unsqueeze(1)  # (B, 1, H, W)
    return pred_error


def compute_normal_metrics(total_normal_errors):
    total_normal_errors = total_normal_errors.detach().cpu().numpy()
    num_pixels = total_normal_errors.shape[0]

    metrics = {
        "mean": np.average(total_normal_errors),
        "median": np.median(total_normal_errors),
        "rmse": np.sqrt(np.sum(total_normal_errors * total_normal_errors) / num_pixels),
        "a1": 100.0 * (np.sum(total_normal_errors < 5) / num_pixels),
        "a2": 100.0 * (np.sum(total_normal_errors < 7.5) / num_pixels),
        "a3": 100.0 * (np.sum(total_normal_errors < 11.25) / num_pixels),
        "a4": 100.0 * (np.sum(total_normal_errors < 22.5) / num_pixels),
        "a5": 100.0 * (np.sum(total_normal_errors < 30) / num_pixels),
    }

    return metrics


def pad_input(orig_H, orig_W):
    if orig_W % 32 == 0:
        l = 0
        r = 0
    else:
        new_W = 32 * ((orig_W // 32) + 1)
        l = (new_W - orig_W) // 2
        r = (new_W - orig_W) - l

    if orig_H % 32 == 0:
        t = 0
        b = 0
    else:
        new_H = 32 * ((orig_H // 32) + 1)
        t = (new_H - orig_H) // 2
        b = (new_H - orig_H) - t
    return l, r, t, b


def get_intrins_from_fov(new_fov, H, W, device):
    # NOTE: top-left pixel should be (0,0)
    if W >= H:
        new_fu = (W / 2.0) / np.tan(np.deg2rad(new_fov / 2.0))
        new_fv = (W / 2.0) / np.tan(np.deg2rad(new_fov / 2.0))
    else:
        new_fu = (H / 2.0) / np.tan(np.deg2rad(new_fov / 2.0))
        new_fv = (H / 2.0) / np.tan(np.deg2rad(new_fov / 2.0))

    new_cu = (W / 2.0) - 0.5
    new_cv = (H / 2.0) - 0.5

    new_intrins = torch.tensor(
        [[new_fu, 0, new_cu], [0, new_fv, new_cv], [0, 0, 1]],
        dtype=torch.float32,
        device=device,
    )

    return new_intrins


def get_intrins_from_txt(intrins_path, device):
    # NOTE: top-left pixel should be (0,0)
    with open(intrins_path, "r") as f:
        intrins_ = f.readlines()[0].split()[0].split(",")
        intrins_ = [float(i) for i in intrins_]
        fx, fy, cx, cy = intrins_

    intrins = torch.tensor(
        [[fx, 0, cx], [0, fy, cy], [0, 0, 1]], dtype=torch.float32, device=device
    )

    return intrins



from pathlib import Path
from argparse import ArgumentParser
import rerun as rr
import mmcv
import cv2
import numpy as np
from typing import get_args

from monopriors.relative_depth_models import (
    get_relative_predictor,
    RelativeDepthPrediction,
    RELATIVE_PREDICTORS,
)
from monopriors.relative_depth_models.base_relative_depth import BaseRelativePredictor
from monopriors.rr_logging_utils import log_relative_pred


def resize_image(image: np.ndarray, max_dim: int = 1024) -> np.ndarray:
    current_dim = max(image.shape[0], image.shape[1])
    if current_dim > max_dim:
        scale_factor = max_dim / current_dim
        image = mmcv.imrescale(img=image, scale=scale_factor)
    return image


def relative_depth_from_img(
    image_path: Path, depth_predictor_name: RELATIVE_PREDICTORS
) -> None:
    parent_log_path = Path("world")
    bgr_hw3 = cv2.imread(str(image_path))
    rgb_hw3 = cv2.cvtColor(bgr_hw3, cv2.COLOR_BGR2RGB)

    max_dim = 1024 // 2
    rgb_hw3 = resize_image(rgb_hw3, max_dim)

    predictor: BaseRelativePredictor = get_relative_predictor(depth_predictor_name)(
        device="cuda"
    )
    relative_pred: RelativeDepthPrediction = predictor.__call__(rgb=rgb_hw3, K_33=None)
    rr.log(f"{parent_log_path}", rr.ViewCoordinates.RDF, timeless=True)
    log_relative_pred(parent_log_path, relative_pred, rgb_hw3)


if __name__ == "__main__":
    parser = ArgumentParser()
    parser.add_argument("--image-path", type=Path, default="data/39703866203.jpg")
    parser.add_argument(
        "--depth-predictor-name",
        choices=get_args(RELATIVE_PREDICTORS),
        default="DepthAnythingV2Predictor",
    )
    rr.script_add_args(parser)
    args = parser.parse_args()
    rr.script_setup(args, "depth view")
    relative_depth_from_img(args.image_path, args.depth_predictor_name)
    rr.script_teardown(args)



import gradio as gr
from monopriors.gradio_ui.depth_inference_ui import depth_inference_block
from monopriors.gradio_ui.depth_compare_ui import relative_compare_block

title = "# Depth Comparison"
description1 = """Demo to help compare different depth models. Including both Scale | Shift Invariant and Metric Depth types."""
description2 = """Invariant models mean they have no true scale and are only relative, where as Metric models have a true scale and are absolute (meters)."""
description3 = """Checkout the [Github Repo](https://github.com/pablovela5620/monoprior) [![GitHub Repo stars](https://img.shields.io/github/stars/pablovela5620/monoprior)](https://github.com/pablovela5620/monoprior)"""
model_load_status: str = "Models loaded and ready to use!"

with gr.Blocks() as demo:
    gr.Markdown(title)
    gr.Markdown(description1)
    gr.Markdown(description2)
    gr.Markdown(description3)
    gr.Markdown("### Depth Prediction demo")
    with gr.Tab(label="Depth Comparison"):
        relative_compare_block.render()
    with gr.Tab(label="Depth Inference"):
        depth_inference_block.render()

if __name__ == "__main__":
    demo.queue().launch()



import numpy as np
import torch
import torch.nn as nn
import torch.nn.functional as F

from .submodules import (
    Encoder,
    ConvGRU,
    UpSampleBN,
    UpSampleGN,
    RayReLU,
    convex_upsampling,
    get_unfold,
    get_prediction_head,
    INPUT_CHANNELS_DICT,
)
from .utils.rotation import axis_angle_to_matrix


class Decoder(nn.Module):
    def __init__(self, output_dims, B=5, NF=2048, BN=False, downsample_ratio=8):
        super(Decoder, self).__init__()
        input_channels = INPUT_CHANNELS_DICT[B]
        output_dim, feature_dim, hidden_dim = output_dims
        features = bottleneck_features = NF
        self.downsample_ratio = downsample_ratio

        UpSample = UpSampleBN if BN else UpSampleGN
        self.conv2 = nn.Conv2d(
            bottleneck_features + 2, features, kernel_size=1, stride=1, padding=0
        )
        self.up1 = UpSample(
            skip_input=features // 1 + input_channels[1] + 2,
            output_features=features // 2,
            align_corners=False,
        )
        self.up2 = UpSample(
            skip_input=features // 2 + input_channels[2] + 2,
            output_features=features // 4,
            align_corners=False,
        )

        # prediction heads
        i_dim = features // 4
        h_dim = 128
        self.normal_head = get_prediction_head(i_dim + 2, h_dim, output_dim)
        self.feature_head = get_prediction_head(i_dim + 2, h_dim, feature_dim)
        self.hidden_head = get_prediction_head(i_dim + 2, h_dim, hidden_dim)

    def forward(self, features, uvs):
        _, _, x_block2, x_block3, x_block4 = (
            features[4],
            features[5],
            features[6],
            features[8],
            features[11],
        )
        uv_32, uv_16, uv_8 = uvs

        x_d0 = self.conv2(torch.cat([x_block4, uv_32], dim=1))
        x_d1 = self.up1(x_d0, torch.cat([x_block3, uv_16], dim=1))
        x_feat = self.up2(x_d1, torch.cat([x_block2, uv_8], dim=1))
        x_feat = torch.cat([x_feat, uv_8], dim=1)

        normal = self.normal_head(x_feat)
        normal = F.normalize(normal, dim=1)
        f = self.feature_head(x_feat)
        h = self.hidden_head(x_feat)
        return normal, f, h


class DSINE(nn.Module):
    def __init__(self):
        super(DSINE, self).__init__()
        self.downsample_ratio = 8
        self.ps = 5  # patch size
        self.num_iter = 5  # num iterations

        # define encoder
        self.encoder = Encoder(B=5, pretrained=True)

        # define decoder
        self.output_dim = output_dim = 3
        self.feature_dim = feature_dim = 64
        self.hidden_dim = hidden_dim = 64
        self.decoder = Decoder(
            [output_dim, feature_dim, hidden_dim], B=5, NF=2048, BN=False
        )

        # ray direction-based ReLU
        self.ray_relu = RayReLU(eps=1e-2)

        # pixel_coords (1, 3, H, W)
        # NOTE: this is set to some arbitrarily high number,
        # if your input is 2000+ pixels wide/tall, increase these values
        h = 2000
        w = 2000
        pixel_coords = np.ones((3, h, w)).astype(np.float32)
        x_range = np.concatenate([np.arange(w).reshape(1, w)] * h, axis=0)
        y_range = np.concatenate([np.arange(h).reshape(h, 1)] * w, axis=1)
        pixel_coords[0, :, :] = x_range + 0.5
        pixel_coords[1, :, :] = y_range + 0.5
        self.pixel_coords = torch.from_numpy(pixel_coords).unsqueeze(0)

        # define ConvGRU cell
        self.gru = ConvGRU(hidden_dim=hidden_dim, input_dim=feature_dim + 2, ks=self.ps)

        # padding used during NRN
        self.pad = (self.ps - 1) // 2

        # prediction heads
        self.prob_head = get_prediction_head(
            self.hidden_dim + 2, 64, self.ps * self.ps
        )  # weights assigned for each nghbr pixel
        self.xy_head = get_prediction_head(
            self.hidden_dim + 2, 64, self.ps * self.ps * 2
        )  # rotation axis for each nghbr pixel
        self.angle_head = get_prediction_head(
            self.hidden_dim + 2, 64, self.ps * self.ps
        )  # rotation angle for each nghbr pixel

        # prediction heads - weights used for upsampling the coarse resolution output
        self.up_prob_head = get_prediction_head(
            self.hidden_dim + 2, 64, 9 * self.downsample_ratio * self.downsample_ratio
        )

    def get_ray(self, intrins, H, W, orig_H, orig_W, return_uv=False):
        B, _, _ = intrins.shape
        fu = intrins[:, 0, 0][:, None, None] * (W / orig_W)
        cu = intrins[:, 0, 2][:, None, None] * (W / orig_W)
        fv = intrins[:, 1, 1][:, None, None] * (H / orig_H)
        cv = intrins[:, 1, 2][:, None, None] * (H / orig_H)

        # (B, 2, H, W)
        ray = self.pixel_coords[:, :, :H, :W].repeat(B, 1, 1, 1)
        ray[:, 0, :, :] = (ray[:, 0, :, :] - cu) / fu
        ray[:, 1, :, :] = (ray[:, 1, :, :] - cv) / fv

        if return_uv:
            return ray[:, :2, :, :]
        else:
            return F.normalize(ray, dim=1)

    def upsample(self, h, pred_norm, uv_8):
        up_mask = self.up_prob_head(torch.cat([h, uv_8], dim=1))
        up_pred_norm = convex_upsampling(pred_norm, up_mask, self.downsample_ratio)
        up_pred_norm = F.normalize(up_pred_norm, dim=1)
        return up_pred_norm

    def refine(self, h, feat_map, pred_norm, intrins, orig_H, orig_W, uv_8, ray_8):
        B, C, H, W = pred_norm.shape
        fu = intrins[:, 0, 0][:, None, None, None] * (W / orig_W)  # (B, 1, 1, 1)
        cu = intrins[:, 0, 2][:, None, None, None] * (W / orig_W)
        fv = intrins[:, 1, 1][:, None, None, None] * (H / orig_H)
        cv = intrins[:, 1, 2][:, None, None, None] * (H / orig_H)

        h_new = self.gru(h, feat_map)

        # get nghbr prob (B, 1, ps*ps, h, w)
        nghbr_prob = self.prob_head(torch.cat([h_new, uv_8], dim=1)).unsqueeze(1)
        nghbr_prob = torch.sigmoid(nghbr_prob)

        # get nghbr normals (B, 3, ps*ps, h, w)
        nghbr_normals = get_unfold(pred_norm, ps=self.ps, pad=self.pad)

        # get nghbr xy (B, 2, ps*ps, h, w)
        nghbr_xys = self.xy_head(torch.cat([h_new, uv_8], dim=1))
        nghbr_xs, nghbr_ys = torch.split(
            nghbr_xys, [self.ps * self.ps, self.ps * self.ps], dim=1
        )
        nghbr_xys = torch.cat([nghbr_xs.unsqueeze(1), nghbr_ys.unsqueeze(1)], dim=1)
        nghbr_xys = F.normalize(nghbr_xys, dim=1)

        # get nghbr theta (B, 1, ps*ps, h, w)
        nghbr_angle = self.angle_head(torch.cat([h_new, uv_8], dim=1)).unsqueeze(1)
        nghbr_angle = torch.sigmoid(nghbr_angle) * np.pi

        # get nghbr pixel coord (1, 3, ps*ps, h, w)
        nghbr_pixel_coord = get_unfold(
            self.pixel_coords[:, :, :H, :W], ps=self.ps, pad=self.pad
        )

        # nghbr axes (B, 3, ps*ps, h, w)
        nghbr_axes = torch.zeros_like(nghbr_normals)

        du_over_fu = nghbr_xys[:, 0, ...] / fu  # (B, ps*ps, h, w)
        dv_over_fv = nghbr_xys[:, 1, ...] / fv  # (B, ps*ps, h, w)

        term_u = (
            nghbr_pixel_coord[:, 0, ...] + nghbr_xys[:, 0, ...] - cu
        ) / fu  # (B, ps*ps, h, w)
        term_v = (
            nghbr_pixel_coord[:, 1, ...] + nghbr_xys[:, 1, ...] - cv
        ) / fv  # (B, ps*ps, h, w)

        nx = nghbr_normals[:, 0, ...]  # (B, ps*ps, h, w)
        ny = nghbr_normals[:, 1, ...]  # (B, ps*ps, h, w)
        nz = nghbr_normals[:, 2, ...]  # (B, ps*ps, h, w)

        nghbr_delta_z_num = -(du_over_fu * nx + dv_over_fv * ny)
        nghbr_delta_z_denom = term_u * nx + term_v * ny + nz
        nghbr_delta_z_denom[torch.abs(nghbr_delta_z_denom) < 1e-8] = 1e-8 * torch.sign(
            nghbr_delta_z_denom[torch.abs(nghbr_delta_z_denom) < 1e-8]
        )
        nghbr_delta_z = nghbr_delta_z_num / nghbr_delta_z_denom

        nghbr_axes[:, 0, ...] = du_over_fu + nghbr_delta_z * term_u
        nghbr_axes[:, 1, ...] = dv_over_fv + nghbr_delta_z * term_v
        nghbr_axes[:, 2, ...] = nghbr_delta_z
        nghbr_axes = F.normalize(nghbr_axes, dim=1)  # (B, 3, ps*ps, h, w)

        # make sure axes are all valid
        invalid = (
            torch.sum(
                torch.logical_or(
                    torch.isnan(nghbr_axes), torch.isinf(nghbr_axes)
                ).float(),
                dim=1,
            )
            > 0.5
        )  # (B, ps*ps, h, w)
        nghbr_axes[:, 0, ...][invalid] = 0.0
        nghbr_axes[:, 1, ...][invalid] = 0.0
        nghbr_axes[:, 2, ...][invalid] = 0.0

        # nghbr_axes_angle (B, 3, ps*ps, h, w)
        nghbr_axes_angle = nghbr_axes * nghbr_angle
        nghbr_axes_angle = nghbr_axes_angle.permute(
            0, 2, 3, 4, 1
        )  # (B, ps*ps, h, w, 3)
        nghbr_R = axis_angle_to_matrix(nghbr_axes_angle)  # (B, ps*ps, h, w, 3, 3)

        # (B, 3, ps*ps, h, w)
        nghbr_normals_rot = (
            torch.bmm(
                nghbr_R.reshape(B * self.ps * self.ps * H * W, 3, 3),
                nghbr_normals.permute(0, 2, 3, 4, 1)
                .reshape(B * self.ps * self.ps * H * W, 3)
                .unsqueeze(-1),
            )
            .reshape(B, self.ps * self.ps, H, W, 3, 1)
            .squeeze(-1)
            .permute(0, 4, 1, 2, 3)
        )  # (B, 3, ps*ps, h, w)
        nghbr_normals_rot = F.normalize(nghbr_normals_rot, dim=1)

        # ray ReLU
        nghbr_normals_rot = torch.cat(
            [
                self.ray_relu(nghbr_normals_rot[:, :, i, :, :], ray_8).unsqueeze(2)
                for i in range(nghbr_normals_rot.size(2))
            ],
            dim=2,
        )

        # (B, 1, ps*ps, h, w) * (B, 3, ps*ps, h, w)
        pred_norm = torch.sum(nghbr_prob * nghbr_normals_rot, dim=2)  # (B, C, H, W)
        pred_norm = F.normalize(pred_norm, dim=1)

        up_mask = self.up_prob_head(torch.cat([h_new, uv_8], dim=1))
        up_pred_norm = convex_upsampling(pred_norm, up_mask, self.downsample_ratio)
        up_pred_norm = F.normalize(up_pred_norm, dim=1)

        return h_new, pred_norm, up_pred_norm

    def forward(self, img, intrins=None):
        # Step 1. encoder
        features = self.encoder(img)

        # Step 2. get uv encoding
        B, _, orig_H, orig_W = img.shape
        intrins[:, 0, 2] += 0.5
        intrins[:, 1, 2] += 0.5
        uv_32 = self.get_ray(
            intrins, orig_H // 32, orig_W // 32, orig_H, orig_W, return_uv=True
        )
        uv_16 = self.get_ray(
            intrins, orig_H // 16, orig_W // 16, orig_H, orig_W, return_uv=True
        )
        uv_8 = self.get_ray(
            intrins, orig_H // 8, orig_W // 8, orig_H, orig_W, return_uv=True
        )
        ray_8 = self.get_ray(intrins, orig_H // 8, orig_W // 8, orig_H, orig_W)

        # Step 3. decoder - initial prediction
        pred_norm, feat_map, h = self.decoder(features, uvs=(uv_32, uv_16, uv_8))
        pred_norm = self.ray_relu(pred_norm, ray_8)

        # Step 4. add ray direction encoding
        feat_map = torch.cat([feat_map, uv_8], dim=1)

        # iterative refinement
        up_pred_norm = self.upsample(h, pred_norm, uv_8)
        pred_list = [up_pred_norm]
        for i in range(self.num_iter):
            h, pred_norm, up_pred_norm = self.refine(
                h, feat_map, pred_norm.detach(), intrins, orig_H, orig_W, uv_8, ray_8
            )
            pred_list.append(up_pred_norm)
        return pred_list



from .base_normal_model import BaseNormalPredictor, SurfaceNormalPrediction
from .dsine_model import DSineNormalPredictor
from .omni_normal_model import OmniNormalPredictor
from typing import Literal, get_args, Callable

# Define predictor names as a list of strings
NORMAL_PREDICTORS = Literal["DSineNormalPredictor", "OmniNormalPredictor"]

# Use the list to generate the __all__ list
__all__: list[str] = list(get_args(NORMAL_PREDICTORS)) + [
    "SurfaceNormalPrediction",
]


def get_normal_predictor(
    predictor_type: NORMAL_PREDICTORS,
) -> Callable[..., BaseNormalPredictor]:
    match predictor_type:
        case "DSineNormalPredictor":
            return DSineNormalPredictor
        case "OmniNormalPredictor":
            return OmniNormalPredictor
        case _:
            raise ValueError(f"Unknown predictor type: {predictor_type}")



from argparse import ArgumentParser
import rerun as rr
from pathlib import Path
from PIL import Image
from monopriors.monoprior_models import DsineAndUnidepth, MonoPriorPrediction
import numpy as np
from monopriors.data.polycam_data import load_raw_polycam_data, PolycamCameraData
from monopriors.depth_fuser import Open3DFuser
import cv2
import zipfile
from tqdm import tqdm
from jaxtyping import UInt8, UInt16


def log_mono_pred(
    parent_path: Path,
    rgb: UInt8[np.ndarray, "h w 3"],
    mono_pred: MonoPriorPrediction,
    cam_data: PolycamCameraData,
    gt_depth: UInt16[np.ndarray, "h w 1"] | None = None,
):
    cam_path = parent_path / "cam"
    pinhole_path = cam_path / "pinhole"
    h, w, _ = rgb.shape

    rr.log(
        f"{cam_path}",
        rr.Transform3D(
            translation=cam_data.cam_T_world_44[:3, 3],
            mat3x3=cam_data.cam_T_world_44[:3, :3],
            from_parent=True,
        ),
    )
    rr.log(
        f"{pinhole_path}",
        rr.Pinhole(
            image_from_camera=cam_data.K_33,
            width=w,
            height=h,
            camera_xyz=rr.ViewCoordinates.RDF,
        ),
    )

    rr.log(f"{pinhole_path}/image", rr.Image(rgb))
    rr.log(
        f"{pinhole_path}/depth",
        rr.DepthImage(mono_pred.metric_pred.depth_meters, meter=1.0),
    )
    rr.log(f"{pinhole_path}/normal", rr.Image(mono_pred.normal_pred.normal_hw3))
    # rr.log(
    #     f"{pinhole_path}/normal_conf",
    #     rr.DepthImage(mono_pred.normal_pred.confidence_hw1),
    # )
    if gt_depth is not None:
        rr.log(f"{pinhole_path}/gt_depth", rr.DepthImage(gt_depth, meter=1000))
        # diff_depth_l1 = np.abs((depth_np.squeeze() - gt_depth))
        # normalize to 0-255
        # diff_depth_l1 = (diff_depth_l1 / diff_depth_l1.max() * 255).astype(np.uint8)
        # rr.log(f"{pinhole_path}/depth_error_l1", rr.Image(diff_depth_l1))


def extract_zip(zip_path: Path, extract_dir: Path) -> None:
    with zipfile.ZipFile(zip_path, "r") as zip_ref:
        zip_ref.extractall(extract_dir)


def main(zip_path: Path) -> None:
    print(zip_path)
    extract_dir: Path = zip_path.parent / zip_path.stem
    print(extract_dir)

    if zip_path.is_file() and zip_path.exists() and (not extract_dir.exists()):
        extract_zip(zip_path, extract_dir=extract_dir)
        final_extract_dir: Path = extract_dir / extract_dir.name
    else:
        final_extract_dir: Path = extract_dir

    print(final_extract_dir)

    image_dir: Path = final_extract_dir / "corrected_images"
    camera_dir: Path = final_extract_dir / "corrected_cameras"
    depth_dir: Path = (
        final_extract_dir / "depth" if (final_extract_dir / "depth").exists() else None
    )

    assert (
        image_dir.exists() and image_dir.is_dir()
    ), f"Image directory not found: {image_dir}"
    assert (
        camera_dir.exists() and camera_dir.is_dir()
    ), f"Camera directory not found: {camera_dir}"

    image_paths: list[Path] = sorted(image_dir.glob("*.jpg"))
    camera_paths: list[Path] = sorted(camera_dir.glob("*.json"))
    if depth_dir is not None:
        assert depth_dir.exists() and depth_dir.is_dir(), "Depth directory not found"
        depth_paths: list[Path] = sorted(depth_dir.glob("*.png"))
        assert len(image_paths) == len(depth_paths), "Image and depth mismatch"

    model = DsineAndUnidepth()
    gt_fuser = Open3DFuser()
    pred_fuser = Open3DFuser()

    parent_path: Path = Path("world")
    rr.log(f"{parent_path}", rr.ViewCoordinates.RUB, timeless=True)

    height, width, _ = np.array(Image.open(image_paths[0])).shape

    pbar = tqdm(zip(image_paths, camera_paths), total=len(image_paths))
    for idx, (image_path, camera_path) in enumerate(pbar):
        rr.set_time_sequence("timestep", idx)
        assert image_path.stem == camera_path.stem, "Image and camera mismatch"
        rgb_hw3: UInt8[np.ndarray, "h w 3"] = np.array(Image.open(image_path))
        # load depth from polycam if available
        if depth_dir is not None:
            # depth of shape 256x192
            gt_depth_hw = cv2.imread(str(depth_paths[idx]), cv2.IMREAD_ANYDEPTH)
            # upscale to image size
            gt_depth_hw: UInt16[np.ndarray, "h w"] = cv2.resize(
                gt_depth_hw,
                (width, height),
                interpolation=cv2.INTER_NEAREST,
            )

        cam_data: PolycamCameraData = load_raw_polycam_data(camera_path)

        pred: MonoPriorPrediction = model.__call__(rgb_hw3, cam_data.K_33)
        # convert to mm and Uint16
        depth_hw1: UInt16[np.ndarray, "h w 1"] = (
            pred.metric_pred.depth_meters * 1000
        ).astype(np.uint16)

        gt_fuser.fuse_frames(
            gt_depth_hw, cam_data.K_33, cam_data.cam_T_world_44, rgb_hw3
        )
        pred_fuser.fuse_frames(
            depth_hw1.squeeze(), cam_data.K_33, cam_data.cam_T_world_44, rgb_hw3
        )

        log_mono_pred(
            parent_path=parent_path,
            rgb=rgb_hw3,
            mono_pred=pred,
            cam_data=cam_data,
            gt_depth=gt_depth_hw if depth_dir is not None else None,
        )
    gt_mesh = gt_fuser.get_mesh()
    gt_mesh.compute_vertex_normals()

    pred_mesh = pred_fuser.get_mesh()
    pred_mesh.compute_vertex_normals()

    rr.log(
        f"{parent_path}/gt_mesh",
        rr.Mesh3D(
            vertex_positions=gt_mesh.vertices,
            triangle_indices=gt_mesh.triangles,
            vertex_normals=gt_mesh.vertex_normals,
            vertex_colors=gt_mesh.vertex_colors,
        ),
    )

    rr.log(
        f"{parent_path}/pred_mesh",
        rr.Mesh3D(
            vertex_positions=pred_mesh.vertices,
            triangle_indices=pred_mesh.triangles,
            vertex_normals=pred_mesh.vertex_normals,
            vertex_colors=pred_mesh.vertex_colors,
        ),
    )


if __name__ == "__main__":
    parser = ArgumentParser()
    parser.add_argument(
        "--zip-path",
        type=Path,
        required=True,
        help="Path to the image for inference",
    )
    rr.script_add_args(parser)

    args = parser.parse_args()
    rr.script_setup(args, "monopriors inference")
    main(args.zip_path)
    rr.script_teardown(args)



import numpy as np
import open3d as o3d
import trimesh
from jaxtyping import Float32, UInt16, UInt8


class DepthFuser:
    def __init__(self, gt_path="", fusion_resolution=0.04, max_fusion_depth=3.0):
        self.fusion_resolution = fusion_resolution
        self.max_fusion_depth = max_fusion_depth


class Open3DFuser(DepthFuser):
    """
    Wrapper class for the open3d fuser.

    This wrapper does not support fusion of tensors with higher than batch 1.
    """

    def __init__(
        self,
        gt_path="",
        fusion_resolution=0.04,
        max_fusion_depth=3,
    ):
        super().__init__(
            gt_path,
            fusion_resolution,
            max_fusion_depth,
        )

        self.fusion_max_depth = max_fusion_depth

        voxel_size = fusion_resolution * 100
        self.volume = o3d.pipelines.integration.ScalableTSDFVolume(
            voxel_length=float(voxel_size) / 100,
            sdf_trunc=3 * float(voxel_size) / 100,
            color_type=o3d.pipelines.integration.TSDFVolumeColorType.RGB8,
        )

    def fuse_frames(
        self,
        depth_hw: UInt16[np.ndarray, "h w"],
        K_33: Float32[np.ndarray, "3 3"],
        cam_T_world_44: Float32[np.ndarray, "4 4"],
        rgb_hw3: UInt8[np.ndarray, "h w 3"],
    ) -> None:
        height = depth_hw.shape[0]
        width = depth_hw.shape[1]

        rgbd = o3d.geometry.RGBDImage.create_from_color_and_depth(
            o3d.geometry.Image(rgb_hw3),
            o3d.geometry.Image(depth_hw),
            depth_scale=1000.0,
            depth_trunc=self.fusion_max_depth,
            convert_rgb_to_intensity=False,
        )

        self.volume.integrate(
            rgbd,
            o3d.camera.PinholeCameraIntrinsic(
                width=width,
                height=height,
                fx=K_33[0, 0],
                fy=K_33[1, 1],
                cx=K_33[0, 2],
                cy=K_33[1, 2],
            ),
            cam_T_world_44,
        )

    def export_mesh(self, path, use_marching_cubes_mask=None):
        o3d.io.write_triangle_mesh(path, self.volume.extract_triangle_mesh())

    def get_mesh(self, export_single_mesh=None, convert_to_trimesh=False):
        mesh = self.volume.extract_triangle_mesh()

        if convert_to_trimesh:
            mesh = trimesh.Trimesh(vertices=mesh.vertices, faces=mesh.triangles)

        return mesh



import gradio as gr
import numpy as np
import torch
from monopriors.relative_depth_models import (
    RelativeDepthPrediction,
    get_relative_predictor,
    RELATIVE_PREDICTORS,
)
from monopriors.metric_depth_models import (
    METRIC_PREDICTORS,
    get_metric_predictor,
    MetricDepthPrediction,
)
from monopriors.rr_logging_utils import (
    log_relative_pred,
    create_compare_depth_blueprint,
    log_metric_pred,
)
from monopriors.depth_utils import estimate_intrinsics
import rerun as rr
from gradio_rerun import Rerun
from pathlib import Path
from typing import Literal, get_args

from jaxtyping import UInt8, Float32
import mmcv
from tqdm import tqdm
import gc


try:
    import spaces  # type: ignore
    IN_SPACES = True
except ImportError:
    print("Not running on Zero")
    IN_SPACES = False

model_load_status: str = "Models loaded and ready to use!"
DEVICE: Literal["cuda"] | Literal["cpu"] = (
    "cuda" if torch.cuda.is_available() else "cpu"
)
MODELS_TO_SKIP: list[str] = []

def predict_depth(
    predictor, model_type: Literal["Relative", "Metric"], rgb: UInt8[np.ndarray, "h w 3"]
) -> RelativeDepthPrediction | MetricDepthPrediction:
    if model_type == "Relative":
        relative_pred: RelativeDepthPrediction = predictor(device=DEVICE).__call__(rgb, None)
        del predictor
        gc.collect()
        torch.cuda.empty_cache()
        return relative_pred
    elif model_type == "Metric":
        K_33:Float32[np.ndarray, "3 3"] = estimate_intrinsics(H=rgb.shape[0], W=rgb.shape[1])
        metric_pred: MetricDepthPrediction = predictor(device=DEVICE).__call__(rgb, K_33=K_33)
        del predictor
        gc.collect()
        torch.cuda.empty_cache()
        return metric_pred


if IN_SPACES:
    predict_depth = spaces.GPU(predict_depth)
    # remove any model that fails on zerogpu spaces
    MODELS_TO_SKIP.extend(["Metric3DRelativePredictor", "Metric3DPredictor"])


@rr.thread_local_stream("depth")
def on_submit(
    rgb: UInt8[np.ndarray, "h w 3"] | None,
    remove_flying_pixels: bool,
    depth_map_threshold: float,
    model_type:Literal["Metric", "Relative"],
    model_1_name: RELATIVE_PREDICTORS | METRIC_PREDICTORS,
    model_2_name: RELATIVE_PREDICTORS | METRIC_PREDICTORS,
    progress=gr.Progress(track_tqdm=True),
)  -> bytes:
    stream: rr.BinaryStream = rr.binary_stream()
    model_names = [model_1_name, model_2_name]
    blueprint = create_compare_depth_blueprint(model_names)
    rr.send_blueprint(blueprint)

    # resize the image to have a max dim of 1024
    max_dim:int = 1024
    height, width, _ = rgb.shape
    current_dim = max(height, width)
    if current_dim > max_dim:
        scale_factor = max_dim / current_dim
        rgb = mmcv.imrescale(img=rgb, scale=scale_factor)

    for model_name in tqdm(model_names, desc="Loading Model and Predicting Depth"):
        # get the name of the model
        parent_log_path = Path(f"{model_name}")
        rr.log(f"{parent_log_path}", rr.ViewCoordinates.RDF, timeless=True)
        if model_type == "Metric":
            predictor = get_metric_predictor(model_name)
            metric_pred: MetricDepthPrediction = predict_depth(predictor, model_type, rgb)
            log_metric_pred(
                parent_log_path=parent_log_path,
                metric_pred=metric_pred,
                rgb_hw3=rgb,
                remove_flying_pixels=remove_flying_pixels,
                depth_edge_threshold=depth_map_threshold
            )
        elif model_type == "Relative":
            predictor = get_relative_predictor(model_name)
            relative_pred: RelativeDepthPrediction = predict_depth(predictor,model_type, rgb)

            log_relative_pred(
                parent_log_path=parent_log_path,
                relative_pred=relative_pred,
                rgb_hw3=rgb,
                remove_flying_pixels=remove_flying_pixels,
                depth_edge_threshold=depth_map_threshold,
            )

    return stream.read()


with gr.Blocks() as relative_compare_block:
    with gr.Row():
        input_image = gr.Image(
            label="Input Image",
            type="numpy",
            height=300,
        )
        with gr.Column():
            with gr.Row():
                remove_flying_pixels = gr.Checkbox(
                    label="Remove Flying Pixels",
                    value=True,
                    interactive=True,
                )
                depth_map_threshold = gr.Slider(
                    label="⬇️ number == more pruning ⬆️ less pruning",
                    minimum=0.05,
                    maximum=0.95,
                    step=0.05,
                    value=0.1,
                )
            with gr.Row():
                model_1_dropdown = gr.Dropdown(
                    choices=list(get_args(RELATIVE_PREDICTORS)),
                    label="Model1",
                    value="DepthAnythingV2Predictor",
                )
                model_2_dropdown = gr.Dropdown(
                    choices=list(get_args(RELATIVE_PREDICTORS)),
                    label="Model2",
                    value="DepthAnythingV1Predictor",
                )
            with gr.Row():
                model_type = gr.Radio(
                    choices=["Metric", "Relative"],
                    value="Relative",
                )
                model_status = gr.Textbox(
                    label="Model Status",
                    value=model_load_status,
                    interactive=False,
                )

    with gr.Row():
        submit = gr.Button(value="Compare Depth")
    rr_viewer = Rerun(
        streaming=False,
        height=800
    )

    submit.click(
        on_submit,
        inputs=[
            input_image,
            remove_flying_pixels,
            depth_map_threshold,
            model_type,
            model_1_dropdown,
            model_2_dropdown
        ],
        outputs=[rr_viewer],
    )

    def change_dropdown(model_type: Literal["Metric", "Relative"]) -> tuple[gr.Dropdown, gr.Dropdown]:
        choices = list(get_args(METRIC_PREDICTORS)) if model_type == "Metric" else list(get_args(RELATIVE_PREDICTORS))
        model_1_dropdown = gr.Dropdown(
            choices=choices,
            label="Model1",
            value="UniDepthMetricPredictor" if model_type == "Metric" else "DepthAnythingV2Predictor",
        )
        model_2_dropdown = gr.Dropdown(
            choices=choices,
            label="Model2",
            value="Metric3DPredictor" if model_type == "Metric" else "UniDepthRelativePredictor",
        )
        return model_1_dropdown, model_2_dropdown
        
    model_type.input(
        fn=change_dropdown,
        inputs=model_type,
        outputs=[model_1_dropdown, model_2_dropdown],
    )

    # get all jpegs in examples path
    examples_paths = Path("examples").glob("*.jpeg")
    # set the examples to be the sorted list of input parameterss (path, remove_flying_pixels, depth_map_threshold)
    examples_list = sorted([[str(path)] for path in examples_paths])
    examples = gr.Examples(
        examples=examples_list,
        inputs=[input_image,
            remove_flying_pixels,
            depth_map_threshold,
            model_type,
            model_1_dropdown,
            model_2_dropdown],
        outputs=[rr_viewer],
        fn=on_submit,
        cache_examples=False,
    )


import torch
import numpy as np


# NOTE: from PyTorch3D
def axis_angle_to_quaternion(axis_angle: torch.Tensor) -> torch.Tensor:
    """
    Convert rotations given as axis/angle to quaternions.

    Args:
        axis_angle: Rotations given as a vector in axis angle form,
            as a tensor of shape (..., 3), where the magnitude is
            the angle turned anticlockwise in radians around the
            vector's direction.

    Returns:
        quaternions with real part first, as tensor of shape (..., 4).
    """
    angles = torch.norm(axis_angle, p=2, dim=-1, keepdim=True)
    half_angles = angles * 0.5
    eps = 1e-6
    small_angles = angles.abs() < eps
    sin_half_angles_over_angles = torch.empty_like(angles)
    sin_half_angles_over_angles[~small_angles] = (
        torch.sin(half_angles[~small_angles]) / angles[~small_angles]
    )
    # for x small, sin(x/2) is about x/2 - (x/2)^3/6
    # so sin(x/2)/x is about 1/2 - (x*x)/48
    sin_half_angles_over_angles[small_angles] = (
        0.5 - (angles[small_angles] * angles[small_angles]) / 48
    )
    quaternions = torch.cat(
        [torch.cos(half_angles), axis_angle * sin_half_angles_over_angles], dim=-1
    )
    return quaternions


# NOTE: from PyTorch3D
def quaternion_to_matrix(quaternions: torch.Tensor) -> torch.Tensor:
    """
    Convert rotations given as quaternions to rotation matrices.

    Args:
        quaternions: quaternions with real part first,
            as tensor of shape (..., 4).

    Returns:
        Rotation matrices as tensor of shape (..., 3, 3).
    """
    r, i, j, k = torch.unbind(quaternions, -1)
    # pyre-fixme[58]: `/` is not supported for operand types `float` and `Tensor`.
    two_s = 2.0 / (quaternions * quaternions).sum(-1)

    o = torch.stack(
        (
            1 - two_s * (j * j + k * k),
            two_s * (i * j - k * r),
            two_s * (i * k + j * r),
            two_s * (i * j + k * r),
            1 - two_s * (i * i + k * k),
            two_s * (j * k - i * r),
            two_s * (i * k - j * r),
            two_s * (j * k + i * r),
            1 - two_s * (i * i + j * j),
        ),
        -1,
    )
    return o.reshape(quaternions.shape[:-1] + (3, 3))


# NOTE: from PyTorch3D
def axis_angle_to_matrix(axis_angle: torch.Tensor) -> torch.Tensor:
    """
    Convert rotations given as axis/angle to rotation matrices.

    Args:
        axis_angle: Rotations given as a vector in axis angle form,
            as a tensor of shape (..., 3), where the magnitude is
            the angle turned anticlockwise in radians around the
            vector's direction.

    Returns:
        Rotation matrices as tensor of shape (..., 3, 3).
    """
    return quaternion_to_matrix(axis_angle_to_quaternion(axis_angle))
